@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -56,8 +56,6 @@ typedef struct {
 	gchar *name;
 	gchar *type;
 	gchar *value;
-	/* gchar *mtime; */
-	/* gchar *muser; */
 } GdaConfigEntry;
 
 typedef struct {
@@ -72,10 +70,14 @@ typedef struct {
 } GdaConfigClient;
 
 static GdaConfigClient *config_client = NULL;
-static gboolean         can_modif_global_conf; /* TRUE if the current user can modify the system wide config file */
+static gboolean         can_modify_global_conf; /* TRUE if the current user can modify the system wide config file */
 static gboolean         lock_write_notify = FALSE;
 static void             do_notify (const gchar *path);
 
+/* if @dsn_list_only_in_mem is TRUE, then the DSN list is not written to any file, but
+ * is just stored in memory, in @dsn_list_contents */
+static gboolean  dsn_list_only_in_mem = FALSE;
+
 /*
  * FAM delcarations and static variables
  */
@@ -129,21 +131,21 @@ gda_config_read_entries (xmlNodePtr cur)
 	list = NULL;
 	cur = cur->xmlChildrenNode;
 	while (cur != NULL){
-		if (!strcmp(cur->name, "entry")){
+		if (!strcmp((gchar*)cur->name, "entry")){
 			entry = g_new0 (GdaConfigEntry, 1);
-			entry->name = xmlGetProp(cur, "name");
+			entry->name = (gchar*)xmlGetProp(cur, (xmlChar*)"name");
 			if (entry->name == NULL){
 				g_warning ("NULL 'name' in an entry");
 				entry->name = g_strdup ("");
 			}
 
-			entry->type =  xmlGetProp(cur, "type");
+			entry->type = (gchar*)xmlGetProp(cur, (xmlChar*)"type");
 			if (entry->type == NULL){
 				g_warning ("NULL 'type' in an entry");
 				entry->type = g_strdup ("");
 			}
 
-			entry->value =  xmlGetProp(cur, "value");
+			entry->value = (gchar*)xmlGetProp(cur, (xmlChar*)"value");
 			if (entry->value == NULL){
 				g_warning ("NULL 'value' in an entry");
 				entry->value = g_strdup ("");
@@ -151,7 +153,7 @@ gda_config_read_entries (xmlNodePtr cur)
 
 			list = g_list_append (list, entry);
 		} else {
-			g_warning ("'entry' expected, got '%s'. Ignoring...", cur->name);
+			g_warning ("'entry' expected, got '%s'. Ignoring...", (gchar*)cur->name);
 		}
 		cur = cur->next;
 	}
@@ -212,8 +214,8 @@ gda_config_parse_config_file (gchar *buffer, gint len)
 		return NULL;
 	}
 
-	if (strcmp (cur->name, "libgda-config") != 0){
-		g_warning ("root node != 'libgda-config' -> '%s'", cur->name);
+	if (strcmp ((gchar*)cur->name, "libgda-config") != 0){
+		g_warning ("root node != 'libgda-config' -> '%s'", (gchar*)cur->name);
 		xmlFreeDoc (doc);
 		xmlMemSetup (old_free,
 			     old_malloc,
@@ -226,14 +228,14 @@ gda_config_parse_config_file (gchar *buffer, gint len)
 		if (xmlNodeIsText (cur)) 
 			continue;
 
-		if (strcmp (cur->name, "section")) {
-			if (strcmp (cur->name, "comment"))
-				g_warning ("'section' expected, got '%s'. Ignoring...", cur->name);
+		if (strcmp ((gchar*)cur->name, "section")) {
+			if (strcmp ((gchar*)cur->name, "comment"))
+				g_warning ("'section' expected, got '%s'. Ignoring...", (gchar*)cur->name);
 			continue;
 		}
 
 		section = g_new0 (GdaConfigSection, 1);
-		section->path = xmlGetProp (cur, "path");
+		section->path = (gchar*)xmlGetProp (cur, (xmlChar*)"path");
 		if (section->path != NULL &&
 		    !strncmp (section->path, section_path, sp_len)){
 			section->entries = gda_config_read_entries (cur);
@@ -266,10 +268,30 @@ get_config_client ()
 	 *	  wait until the operation finishes
 	 */
 	if (! config_client) {
-		gint len;
-		gchar *full_file;
+		guint len = 0;
+		gchar *full_file = NULL;
 		gchar *user_config = NULL;
-		gboolean has_user_config;
+		gboolean has_user_config = FALSE;
+		gchar *memonly;
+		
+		
+		config_client = g_new0 (GdaConfigClient, 1);
+		xmlKeepBlanksDefault(0);
+
+		/* check if DSN list should only be stored in memory */
+		memonly = getenv ("GDA_DSN_LIST_IN_MEMORY");
+		if (memonly) {
+			gsize length;
+			gchar *init_contents;
+			dsn_list_only_in_mem = TRUE;
+
+			g_print ("** DSN list will remain in memory and will be lost at the end of the session **\n");
+			if (*memonly && g_file_get_contents (memonly, &init_contents, &length, NULL)) {
+				config_client->user = gda_config_parse_config_file (init_contents, length);
+				g_free (init_contents);
+			}
+			return config_client;
+		}
 
 		has_user_config = g_get_home_dir () ? TRUE : FALSE;
 		if (has_user_config)
@@ -281,14 +303,14 @@ get_config_client ()
 
 			file = fopen (LIBGDA_GLOBAL_CONFIG_FILE, "a");
 			if (file) {
-				can_modif_global_conf = TRUE;
+				can_modify_global_conf = TRUE;
 #ifdef GDA_DEBUG_NO
 				g_print ("Can write system wide config file\n");
 #endif
 				fclose (file);
 			}
 			else {
-				can_modif_global_conf = FALSE;
+				can_modify_global_conf = FALSE;
 #ifdef GDA_DEBUG_NO
 				g_print ("Cannot write system wide config file\n");
 #endif			
@@ -337,9 +359,6 @@ get_config_client ()
 			
 		}
 #endif
-		
-		config_client = g_new0 (GdaConfigClient, 1);
-		xmlKeepBlanksDefault(0);
 
 		/*
 		 * load system wide config
@@ -674,10 +693,10 @@ add_xml_entry (xmlNodePtr parent, GdaConfigEntry *entry)
 {
 	xmlNodePtr new_node;
 
-	new_node = xmlNewTextChild (parent, NULL, "entry", NULL);
-	xmlSetProp (new_node, "name", entry->name);
-	xmlSetProp (new_node, "type", entry->type);
-	xmlSetProp (new_node, "value", entry->value);
+	new_node = xmlNewTextChild (parent, NULL, (xmlChar*)"entry", NULL);
+	xmlSetProp (new_node, (xmlChar*)"name", (xmlChar*)entry->name);
+	xmlSetProp (new_node, (xmlChar*)"type", (xmlChar*)entry->type);
+	xmlSetProp (new_node, (xmlChar*)"value", (xmlChar*)entry->value);
 }
 
 static xmlNodePtr
@@ -685,8 +704,8 @@ add_xml_section (xmlNodePtr parent, GdaConfigSection *section)
 {
 	xmlNodePtr new_node;
 
-	new_node = xmlNewTextChild (parent, NULL, "section", NULL);
-	xmlSetProp (new_node, "path", section->path ? section->path : "");
+	new_node = xmlNewTextChild (parent, NULL, (xmlChar*)"section", NULL);
+	xmlSetProp (new_node, (xmlChar*)"path", (xmlChar*)(section->path ? section->path : ""));
 	return new_node;
 }
 
@@ -706,11 +725,15 @@ write_config_file ()
 	if (lock_write_notify)
 		return;
 
+	if (dsn_list_only_in_mem)
+		/* nothing to do */
+		return;
+
 	/* user specific data sources */
 	cfg_client = get_config_client ();
-	doc = xmlNewDoc ("1.0");
+	doc = xmlNewDoc ((xmlChar*)"1.0");
 	g_return_if_fail (doc != NULL);
-	root = xmlNewDocNode (doc, NULL, "libgda-config", NULL);
+	root = xmlNewDocNode (doc, NULL, (xmlChar*)"libgda-config", NULL);
 	xmlDocSetRootElement (doc, root);
 	for (ls = cfg_client->user; ls; ls = ls->next){
 		section = ls->data;
@@ -740,10 +763,10 @@ write_config_file ()
 	xmlFreeDoc (doc);
 
 	/* system wide data sources */
-	if (can_modif_global_conf) {
-		doc = xmlNewDoc ("1.0");
+	if (can_modify_global_conf) {
+		doc = xmlNewDoc ((xmlChar*)"1.0");
 		g_return_if_fail (doc != NULL);
-		root = xmlNewDocNode (doc, NULL, "libgda-config", NULL);
+		root = xmlNewDocNode (doc, NULL, (xmlChar*)"libgda-config", NULL);
 		xmlDocSetRootElement (doc, root);
 		for (ls = cfg_client->global; ls; ls = ls->next){
 			section = ls->data;
@@ -905,7 +928,7 @@ gda_config_set_string (const gchar *path, const gchar *new_value)
 	entry = gda_config_search_entry (cfg_client->user, path, "string");
 	if (!entry) {
 		entry = gda_config_search_entry (cfg_client->global, path, "string");
-		if (entry && !can_modif_global_conf)
+		if (entry && !can_modify_global_conf)
 			return FALSE;
 	}
 	if (entry == NULL){
@@ -959,7 +982,7 @@ gda_config_set_int (const gchar *path, gint new_value)
 	entry = gda_config_search_entry (cfg_client->user, path, "long");
 	if (!entry) {
 		entry = gda_config_search_entry (cfg_client->global, path, "long");
-		if (entry && !can_modif_global_conf)
+		if (entry && !can_modify_global_conf)
 			return FALSE;
 	}
 	if (entry == NULL){
@@ -1014,7 +1037,7 @@ gda_config_set_float (const gchar * path, gdouble new_value)
 	entry = gda_config_search_entry (cfg_client->user, path, "float");
 	if (!entry) {
 		entry = gda_config_search_entry (cfg_client->global, path, "float");
-		if (entry && !can_modif_global_conf)
+		if (entry && !can_modify_global_conf)
 			return FALSE;
 	}
 	if (entry == NULL){
@@ -1070,7 +1093,7 @@ gda_config_set_boolean (const gchar *path, gboolean new_value)
 	entry = gda_config_search_entry (cfg_client->user, path, "bool");
 	if (!entry) {
 		entry = gda_config_search_entry (cfg_client->global, path, "bool");
-		if (entry && !can_modif_global_conf)
+		if (entry && !can_modify_global_conf)
 			return FALSE;
 	}
 	if (entry == NULL){
@@ -1406,6 +1429,129 @@ gda_config_free_list (GList *list)
 	g_list_free (list);
 }
 
+static GList *
+load_providers_from_dir (const gchar *dirname, gboolean recurs)
+{
+	GDir *dir;
+	GError *err = NULL;
+	const gchar *name;
+	GList *list = NULL;
+	
+	/* read the plugin directory */
+#ifdef GDA_DEBUG_NO
+	g_print ("Loading providers in %s\n", dirname);
+#endif
+	dir = g_dir_open (dirname, 0, &err);
+	if (err) {
+		gda_log_error (err->message);
+		g_error_free (err);
+		return NULL;
+	}
+	
+	while ((name = g_dir_read_name (dir))) {
+		GdaProviderInfo *info;
+		GModule *handle;
+		gchar *path;
+		gchar *ext;
+		void (*plugin_init) (const gchar *);
+		const gchar * (* plugin_get_name) (void);
+		const gchar * (* plugin_get_description) (void);
+		gchar * (* plugin_get_dsn_spec) (void);
+			
+		if (recurs) {
+			gchar *cname;
+			cname = g_build_filename (dirname, name, NULL);
+			if (g_file_test (cname, G_FILE_TEST_IS_DIR)) {
+				GList *nlist;
+				nlist = load_providers_from_dir (cname, TRUE);
+				if (nlist)
+					list = g_list_concat (list, nlist);
+			}
+			g_free (cname);
+		}
+
+		ext = g_strrstr (name, ".");
+		if (!ext)
+			continue;
+		if (strcmp (ext + 1, G_MODULE_SUFFIX))
+			continue;
+			
+		path = g_build_path (G_DIR_SEPARATOR_S, dirname,
+				     name, NULL);
+		handle = g_module_open (path, G_MODULE_BIND_LAZY);
+		if (!handle) {
+			g_warning (_("Error: %s"), g_module_error ());
+			g_free (path);
+			continue;
+		}
+			
+		if (g_module_symbol (handle, "plugin_init",
+				     (gpointer *) &plugin_init))
+			plugin_init (dirname);
+
+		g_module_symbol (handle, "plugin_get_name",
+				 (gpointer *) &plugin_get_name);
+		g_module_symbol (handle, "plugin_get_description",
+				 (gpointer *) &plugin_get_description);
+		g_module_symbol (handle, "plugin_get_dsn_spec",
+				 (gpointer *) &plugin_get_dsn_spec);
+			
+		info = g_new0 (GdaProviderInfo, 1);
+		info->location = path;
+			
+		if (plugin_get_name != NULL)
+			info->id = g_strdup (plugin_get_name ());
+		else
+			info->id = g_strdup (name);
+			
+		if (plugin_get_description != NULL)
+			info->description = g_strdup (plugin_get_description ());
+		else
+			info->description = NULL;
+			
+		info->dsn_spec = NULL;
+		info->gda_params = NULL;
+
+		if (plugin_get_dsn_spec) {
+			GError *error = NULL;
+				
+			info->dsn_spec = plugin_get_dsn_spec ();
+			if (info->dsn_spec) {
+				info->gda_params = gda_parameter_list_new_from_spec_string (NULL, info->dsn_spec, &error);
+				if (!info->gda_params) {
+					g_warning ("Invalid format for provider '%s' DSN spec : %s",
+						   info->id,
+						   error ? error->message : "Unknown error");
+					if (error)
+						g_error_free (error);
+				}
+			}
+			else {
+				/* there may be traces of the provider installed but some parts are missing,
+				   forget about that provider... */
+				gda_provider_info_free (info);
+				info = NULL;
+			}
+		}
+		else 
+			g_warning ("Provider '%s' does not provide a DSN spec", info->id);
+			
+		if (info) {
+			list = g_list_append (list, info);
+#ifdef GDA_DEBUG_NO
+			g_print ("Loaded '%s' provider\n", info->id);
+#endif
+		}
+			
+		g_module_close (handle);
+	}
+		
+	/* free memory */
+	g_dir_close (dir);
+
+	return list;
+}
+
 /**
  * gda_config_get_provider_list
  *
@@ -1421,100 +1567,13 @@ gda_config_get_provider_list (void)
 	static GList *prov_list = NULL;
 
 	if (!prov_list) {
-		GDir *dir;
-		GError *err = NULL;
-		const gchar *name;
-		GList *list = NULL;
+		const gchar *from_dir;
 
-		/* read the plugin directory */
-		dir = g_dir_open (LIBGDA_PLUGINDIR, 0, &err);
-		if (err) {
-			gda_log_error (err->message);
-			g_error_free (err);
-			return NULL;
-		}
-		
-		while ((name = g_dir_read_name (dir))) {
-			GdaProviderInfo *info;
-			GModule *handle;
-			gchar *path;
-			gchar *ext;
-			const gchar * (* plugin_get_name) (void);
-			const gchar * (* plugin_get_description) (void);
-			gchar * (* plugin_get_dsn_spec) (void);
-			
-			ext = g_strrstr (name, ".");
-			if (!ext)
-				continue;
-			if (strcmp (ext + 1, G_MODULE_SUFFIX))
-				continue;
-			
-			path = g_build_path (G_DIR_SEPARATOR_S, LIBGDA_PLUGINDIR,
-					     name, NULL);
-			handle = g_module_open (path, G_MODULE_BIND_LAZY);
-			if (!handle) {
-				g_warning (_("Error: %s"), g_module_error ());
-				g_free (path);
-				continue;
-			}
-			
-			g_module_symbol (handle, "plugin_get_name",
-					 (gpointer *) &plugin_get_name);
-			g_module_symbol (handle, "plugin_get_description",
-					 (gpointer *) &plugin_get_description);
-			g_module_symbol (handle, "plugin_get_dsn_spec",
-					 (gpointer *) &plugin_get_dsn_spec);
-			
-			info = g_new0 (GdaProviderInfo, 1);
-			info->location = path;
-			
-			if (plugin_get_name != NULL)
-				info->id = g_strdup (plugin_get_name ());
-			else
-				info->id = g_strdup (name);
-			
-			if (plugin_get_description != NULL)
-				info->description = g_strdup (plugin_get_description ());
-			else
-				info->description = NULL;
-			
-			info->dsn_spec = NULL;
-			info->gda_params = NULL;
-
-			if (plugin_get_dsn_spec) {
-				GError *error = NULL;
-				
-				info->dsn_spec = plugin_get_dsn_spec ();
-				if (info->dsn_spec) {
-					info->gda_params = gda_parameter_list_new_from_spec_string (NULL, info->dsn_spec, &error);
-					if (!info->gda_params) {
-						g_warning ("Invalid format for provider '%s' DSN spec : %s",
-							   info->id,
-							   error ? error->message : "Unknown error");
-						if (error)
-							g_error_free (error);
-					}
-				}
-				else {
-					/* there may be traces of the provider installed but some parts are missing,
-					 forget about that provider... */
-					gda_provider_info_free (info);
-					info = NULL;
-				}
-			}
-			else 
-				g_warning ("Provider '%s' does not provide a DSN spec", info->id);
-			
-			if (info)
-				list = g_list_append (list, info);
-			
-			g_module_close (handle);
-		}
-		
-		/* free memory */
-		g_dir_close (dir);
-
-		prov_list = list;
+		from_dir = getenv ("GDA_PROVIDERS_ROOT_DIR");
+		if (from_dir)
+			prov_list = load_providers_from_dir (from_dir, TRUE);
+		else
+			prov_list = load_providers_from_dir (LIBGDA_PLUGINDIR, FALSE);
 	}
 
 	return prov_list;
@@ -1958,7 +2017,7 @@ gboolean
 gda_config_can_modify_global_config (void)
 {
 	get_config_client ();
-	return can_modif_global_conf;
+	return can_modify_global_conf;
 }
 
 /**
@@ -1993,7 +2052,7 @@ gda_config_save_data_source (const gchar *name,
 	g_return_val_if_fail (name != NULL, FALSE);
 	g_return_val_if_fail (provider != NULL, FALSE);
 
-	if (is_global && !can_modif_global_conf)
+	if (is_global && !can_modify_global_conf)
 		return FALSE;
 
 	/* delay write and nofity */
