@@ -390,7 +390,7 @@ gda_data_model_hash_get_type (void)
 {
 	static GType type = 0;
 
-	if (G_UNLIKELY (type == 0)) {
+	if (!type) {
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelHashClass),
 			(GBaseInitFunc) NULL,
@@ -406,7 +406,6 @@ gda_data_model_hash_get_type (void)
 					       "GdaDataModelHash",
 					       &info, 0);
 	}
-
 	return type;
 }
 
@@ -428,7 +427,7 @@ gda_data_model_hash_new (gint cols)
  * gda_data_model_hash_insert_row
  * @model: the #GdaDataModelHash which is gonna hold the row.
  * @rownum: the number of the row.
- * @row: the row to insert. The model is responsible for freeing it with g_object_unref().
+ * @row: the row to insert. The model is responsible of freeing it!
  *
  * Inserts a @row in the @model.
  */
@@ -518,6 +517,6 @@ gda_data_model_hash_clear (GdaDataModelHash *model)
         for (i = 0; i < model->priv->number_of_hash_table_rows; i++)
                 g_array_append_val (model->priv->row_map, i);
 
-	gda_data_model_reset (GDA_DATA_MODEL (model));
+	/* FIXME: signal changes */
 }
 
