@@ -86,7 +86,7 @@ gda_dict_type_get_type (void)
 {
 	static GType type = 0;
 
-	if (G_UNLIKELY (type == 0)) {
+	if (!type) {
 		static const GTypeInfo info = {
 			sizeof (GdaDictTypeClass),
 			(GBaseInitFunc) NULL,
