@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2003 - 2006 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef __GDA_DICT_H_
@@ -26,7 +26,14 @@
 #include <libgda/gda-value.h>
 #include <libgda/gda-enums.h>
 
-/* #include <libgda/gda-dict-reg-aggregates.h> */
+/* 
+You will need to include these when including gda-dict.h, 
+to use some functions.
+TODO: Avoid this awkwarness:
+#include <libgda/gda-dict-reg-aggregates.h>
+#include <libgda/gda-dict-reg-functions.h>
+*/
+
 
 G_BEGIN_DECLS
 
@@ -71,7 +78,7 @@ struct _GdaDictClass
 	void    (*object_act_changed)   (GdaDict *dict, GdaObject *obj);
 
         void    (*data_update_started)  (GdaDict *dict);
-        void    (*update_progress)      (GdaDict *dict, gchar * msg, guint now, guint total);
+        void    (*update_progress)      (GdaDict *dict, gchar *msg, guint now, guint total);
         void    (*data_update_finished) (GdaDict *dict);
 
 	/* signal that a change in the whole dictionary has occurred */
