@@ -2,28 +2,33 @@
  *
  * Copyright (C) 2003 - 2006 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
+#include <stdarg.h>
 #include <string.h>
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #include <glib/gi18n-lib.h>
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
 #include "gda-parameter-list.h"
+#include "gda-parameter-util.h"
 #include "gda-query.h"
 #include "gda-marshal.h"
 #include "gda-data-model.h"
@@ -55,6 +60,10 @@ static void paramlist_remove_source (GdaParameterList *paramlist, GdaParameterLi
 static void destroyed_param_cb (GdaParameter *param, GdaParameterList *dataset);
 static void changed_param_cb (GdaParameter *param, GdaParameterList *dataset);
 static void restrict_changed_param_cb (GdaParameter *param, GdaParameterList *dataset);
+static void notify_param_cb (GdaParameter *param, GParamSpec *pspec, GdaParameterList *dataset);
+
+static void compute_public_data (GdaParameterList *paramlist);
+static void gda_parameter_list_real_add_param (GdaParameterList *paramlist, GdaParameter *param);
 
 #ifdef GDA_DEBUG
 static void gda_parameter_list_dump                     (GdaParameterList *paramlist, guint offset);
@@ -68,10 +77,19 @@ enum
 {
 	PARAM_CHANGED,
 	PUBLIC_DATA_CHANGED,
+	PARAM_PLUGIN_CHANGED,
+	PARAM_ATTR_CHANGED,
 	LAST_SIGNAL
 };
 
-static gint gda_parameter_list_signals[LAST_SIGNAL] = { 0, 0 };
+/* properties */
+enum
+{
+	PROP_0,
+	PROP_PARAMS
+};
+
+static gint gda_parameter_list_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
 
 
 /* private structure */
@@ -84,7 +102,50 @@ struct _GdaParameterListPrivate
 	GHashTable *param_repl;                /* key = parameter, value = parameter used instead because of similarity */
 };
 
+static void 
+gda_parameter_list_set_property (GObject *object,
+			 guint param_id,
+			 const GValue *value,
+			 GParamSpec *pspec)
+{
+	GdaParameterList* paramlist;
 
+	paramlist = GDA_PARAMETER_LIST (object);
+
+	switch (param_id) {
+		case PROP_PARAMS: 
+		{
+			/* add the parameters */
+			GSList* params = (GSList*) g_value_get_pointer(value);
+			while (params) {
+				gda_parameter_list_real_add_param (paramlist, GDA_PARAMETER (params->data));
+				params = g_slist_next (params);
+			}
+			compute_public_data (paramlist);	
+			break;
+		}
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
+	}
+}
+
+static void
+gda_parameter_list_get_property (GObject *object,
+			 guint param_id,
+			 GValue *value,
+			 GParamSpec *pspec)
+{
+	GdaParameterList* paramlist;
+
+	paramlist = GDA_PARAMETER_LIST (object);
+
+	switch (param_id) {
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
+	}
+}
 
 /* module error */
 GQuark gda_parameter_list_error_quark (void)
@@ -134,7 +195,23 @@ gda_parameter_list_class_init (GdaParameterListClass *class)
 			      G_STRUCT_OFFSET (GdaParameterListClass, param_changed),
 			      NULL, NULL,
 			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
-			      G_TYPE_OBJECT);
+			      GDA_TYPE_PARAMETER);
+	gda_parameter_list_signals[PARAM_PLUGIN_CHANGED] =
+		g_signal_new ("param_plugin_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GdaParameterListClass, param_plugin_changed),
+			      NULL, NULL,
+			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
+			      GDA_TYPE_PARAMETER);
+	gda_parameter_list_signals[PARAM_ATTR_CHANGED] =
+		g_signal_new ("param_attr_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GdaParameterListClass, param_attr_changed),
+			      NULL, NULL,
+			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
+			      GDA_TYPE_PARAMETER);
 	gda_parameter_list_signals[PUBLIC_DATA_CHANGED] =
 		g_signal_new ("public_data_changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -144,8 +221,18 @@ gda_parameter_list_class_init (GdaParameterListClass *class)
 			      gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 	class->param_changed = NULL;
+	class->param_plugin_changed = NULL;
+	class->param_attr_changed = NULL;
 	class->public_data_changed = NULL;
 
+  /* Properties */
+	object_class->set_property = gda_parameter_list_set_property;
+	object_class->get_property = gda_parameter_list_get_property;
+	g_object_class_install_property (object_class, PROP_PARAMS,
+					 g_param_spec_pointer ("params", "GSList of GdaParameters", 
+							       "Parameters the list should contain", (
+								G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)));
+  
 	object_class->dispose = gda_parameter_list_dispose;
 	object_class->finalize = gda_parameter_list_finalize;
 
@@ -175,8 +262,28 @@ gda_parameter_list_init (GdaParameterList *paramlist)
 	paramlist->priv->param_repl = g_hash_table_new (NULL, NULL);
 }
 
-static void compute_public_data (GdaParameterList *paramlist);
-static void gda_parameter_list_real_add_param (GdaParameterList *paramlist, GdaParameter *param);
+/**
+ * _gda_parameter_list_get_dict
+ * @params: a list of #GdaParameter objects
+ *
+ * FIXME: Helper method for wrappers
+ * Returns: a GdaDict object
+ */
+
+GdaDict* _gda_parameter_list_get_dict(GSList *params)
+{
+	GdaDict *dict = NULL;
+	/* get a valid GdaDict object */
+	while (params) {
+		if (!dict)
+			dict = gda_object_get_dict (GDA_OBJECT (params->data));
+		else
+			if (dict != gda_object_get_dict (GDA_OBJECT (params->data)))
+				g_warning ("Several parameters don't relate to the same GdaDict object");
+		params = g_slist_next (params);
+	}
+  return dict;
+}
 
 /**
  * gda_parameter_list_new
@@ -194,34 +301,103 @@ gda_parameter_list_new (GSList *params)
 {
 	GObject *obj;
 	GdaParameterList *paramlist;
-	GSList *list;
-	GdaDict *dict = NULL;
-
-	/* get a valid GdaDict object */
-	list = params;
-	while (list) {
-		if (!dict)
-			dict = gda_object_get_dict (GDA_OBJECT (list->data));
-		else
-			if (dict != gda_object_get_dict (GDA_OBJECT (list->data)))
-				g_warning ("Several parameters don't relate to the same GdaDict object");
-
-		list = g_slist_next (list);
-	}
-
-	obj = g_object_new (GDA_TYPE_PARAMETER_LIST, "dict", dict, NULL);
-        paramlist = GDA_PARAMETER_LIST (obj);
-
-	/* add the parameters */
-	while (params) {
-		gda_parameter_list_real_add_param (paramlist, GDA_PARAMETER (params->data));
-		params = g_slist_next (params);
-	}
-	compute_public_data (paramlist);
 
+	obj = g_object_new (GDA_TYPE_PARAMETER_LIST, "dict", _gda_parameter_list_get_dict(params), 
+											"params", params, NULL);
+  paramlist = GDA_PARAMETER_LIST (obj);
+	
 	return paramlist;
 }
 
+/**
+ * gda_parameter_list_new_inline
+ * @dict: a #GdaDict object, or %NULL
+ * @...: a serie of a (const gchar*) name, (GType) type, and value, terminated by a %NULL
+ *
+ * Creates a new #GdaParameterList containing parameters defined by each triplet in ...
+ * For each triplet (name, Glib type and value), the value must be of the correct type (gchar * if type is G_STRING, ...)
+ *
+ * Returns: a new #GdaParameterList object
+ */ 
+GdaParameterList *
+gda_parameter_list_new_inline (GdaDict *dict, ...)
+{
+	GdaParameterList *plist;
+	GSList *params = NULL;
+	va_list ap;
+	gchar *name;
+
+	g_return_val_if_fail (!dict || GDA_IS_DICT (dict), NULL);
+
+	/* build the list of parameters */
+	va_start  (ap, dict);
+	name = va_arg (ap, char *);
+	while (name) {
+		GType type;
+		GdaParameter *param;
+		GValue *value;
+
+		type = va_arg (ap, GType);
+		param = (GdaParameter *) g_object_new (GDA_TYPE_PARAMETER, 
+						       "dict", ASSERT_DICT (dict), "g_type", type, NULL);
+		gda_object_set_name (GDA_OBJECT (param), name);
+		gda_object_set_id (GDA_OBJECT (param), name);
+
+		value = gda_value_new (type);
+		if (type == G_TYPE_BOOLEAN) 
+			g_value_set_boolean (value, va_arg (ap, int));
+                else if (type == G_TYPE_STRING)
+			g_value_set_string (value, va_arg (ap, gchar *));
+                else if (type == G_TYPE_OBJECT)
+			g_value_set_object (value, va_arg (ap, gpointer));
+		else if (type == G_TYPE_INT)
+			g_value_set_int (value, va_arg (ap, gint));
+		else if (type == G_TYPE_UINT)
+			g_value_set_uint (value, va_arg (ap, guint));
+		else if (type == GDA_TYPE_BINARY)
+			gda_value_set_binary (value, va_arg (ap, GdaBinary *));
+		else if (type == G_TYPE_INT64)
+			g_value_set_int64 (value, va_arg (ap, gint64));
+		else if (type == G_TYPE_UINT64)
+			g_value_set_uint64 (value, va_arg (ap, guint64));
+		else if (type == GDA_TYPE_SHORT)
+			gda_value_set_short (value, va_arg (ap, int));
+		else if (type == GDA_TYPE_USHORT)
+			gda_value_set_ushort (value, va_arg (ap, guint));
+		else if (type == G_TYPE_CHAR)
+			g_value_set_char (value, va_arg (ap, int));
+		else if (type == G_TYPE_UCHAR)
+			g_value_set_uchar (value, va_arg (ap, guint));
+		else if (type == G_TYPE_FLOAT)
+			g_value_set_float (value, va_arg (ap, double));
+		else if (type == G_TYPE_DOUBLE)
+			g_value_set_double (value, va_arg (ap, gdouble));
+		else if (type == GDA_TYPE_NUMERIC)
+			gda_value_set_numeric (value, va_arg (ap, GdaNumeric *));
+		else if (type == G_TYPE_DATE)
+			g_value_set_boxed (value, va_arg (ap, GDate *));
+		else
+			g_warning ("%s() does not handle values of type %s, value will not be assigned.",
+				   __FUNCTION__, g_type_name (type));
+
+		gda_parameter_set_value (param, value);
+		gda_value_free (value);
+		params = g_slist_append (params, param);
+
+		name = va_arg (ap, char *);
+        }
+	va_end (ap);
+
+	/* create the plist */
+	plist = gda_parameter_list_new (params);
+	if (params) {
+		g_slist_foreach (params, (GFunc) g_object_unref, NULL);
+		g_slist_free (params);
+	}
+
+	return plist;
+}
+
 static void
 xml_validity_error_func (void *ctx, const char *msg, ...)
 {
@@ -286,11 +462,14 @@ gda_parameter_list_new_from_spec_string (GdaDict *dict, const gchar *xml_spec, G
                 xmlDoValidityCheckingDefaultValue = 1;
 
                 /* replace the DTD with ours */
-		old_dtd = doc->intSubset;
-                doc->intSubset = gda_paramlist_dtd;
+		if (gda_paramlist_dtd) {
+			old_dtd = doc->intSubset;
+			doc->intSubset = gda_paramlist_dtd;
+		}
 
-                if (! xmlValidateDocument (validc, doc)) {
-			doc->intSubset = old_dtd;
+                if (doc->intSubset && !xmlValidateDocument (validc, doc)) {
+			if (gda_paramlist_dtd)
+				doc->intSubset = old_dtd;
                         xmlFreeDoc (doc);
                         g_free (validc);
 			
@@ -310,14 +489,15 @@ gda_parameter_list_new_from_spec_string (GdaDict *dict, const gchar *xml_spec, G
                         xmlDoValidityCheckingDefaultValue = xmlcheck;
                         return NULL;
                 }
-		doc->intSubset = old_dtd;
+		if (gda_paramlist_dtd)
+			doc->intSubset = old_dtd;
                 xmlDoValidityCheckingDefaultValue = xmlcheck;
                 g_free (validc);
         }
 
 	/* doc is now non NULL */
 	root = xmlDocGetRootElement (doc);
-	if (strcmp (root->name, "data-set-spec") != 0){
+	if (strcmp ((gchar*)root->name, "data-set-spec") != 0){
 		g_set_error (error, GDA_PARAMETER_LIST_ERROR, GDA_PARAMETER_LIST_XML_SPEC_ERROR,
 			     _("Spec's root node != 'data-set-spec': '%s'"), root->name);
 		return NULL;
@@ -373,7 +553,7 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
         if (cnc)
                 prov = gda_connection_get_provider_obj (cnc);
 
-	if (strcmp (xml_spec->name, "parameters") != 0){
+	if (strcmp ((gchar*)xml_spec->name, "parameters") != 0){
 		g_set_error (error, GDA_PARAMETER_LIST_ERROR, GDA_PARAMETER_LIST_XML_SPEC_ERROR,
 			     _("Missing node <parameters>: '%s'"), xml_spec->name);
 		return NULL;
@@ -381,14 +561,14 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 
 	/* Parameters' sources, not mandatory: makes the @sources list */
 	cur = xml_spec->next;
-	while (cur && (xmlNodeIsText (cur) || strcmp (cur->name, "sources"))) 
+	while (cur && (xmlNodeIsText (cur) || strcmp ((gchar*)cur->name, "sources"))) 
 		cur = cur->next; 
-	if (allok && cur && !strcmp (cur->name, "sources")){
+	if (allok && cur && !strcmp ((gchar*)cur->name, "sources")){
 		for (cur = cur->xmlChildrenNode; (cur != NULL) && allok; cur = cur->next) {
 			if (xmlNodeIsText (cur)) 
 				continue;
 
-			if (!strcmp (cur->name, "gda_array")) {
+			if (!strcmp ((gchar*)cur->name, "gda_array")) {
 				GdaDataModel *model;
 				GSList *errors;
 
@@ -403,7 +583,7 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 				}
 				else  {
 					sources = g_slist_prepend (sources, model);
-					str = xmlGetProp(cur, "name");
+					str = (gchar*)xmlGetProp(cur, (xmlChar*)"name");
 					if (str) {
 						gda_object_set_name (GDA_OBJECT (model), str);
 						g_free (str);
@@ -418,7 +598,7 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 		if (xmlNodeIsText (cur)) 
 			continue;
 
-		if (!strcmp (cur->name, "parameter")) {
+		if (!strcmp ((gchar*)cur->name, "parameter")) {
 			GdaParameter *param = NULL;
 			GdaDictType *dtype = NULL;
 			gchar *str, *id;
@@ -428,14 +608,14 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 			xmlChar *xmlstr;
 
 			/* don't care about entries for the wrong locale */
-			this_lang = xmlGetProp (cur, "lang");
-			if (this_lang && strncmp (this_lang, lang, strlen (this_lang))) {
+			this_lang = xmlGetProp(cur, (xmlChar*)"lang");
+			if (this_lang && strncmp ((gchar*)this_lang, lang, strlen ((gchar*)this_lang))) {
 				g_free (this_lang);
 				continue;
 			}
 
 			/* find if there is already a param with the same ID */
-			id = xmlGetProp (cur, "id");
+			id = (gchar*)xmlGetProp(cur, (xmlChar*)"id");
 			list = params;
 			while (list && !param) {
 				g_object_get (G_OBJECT (list->data), "string_id", &str, NULL);
@@ -457,8 +637,8 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 			/* find data type and create GdaParameter */
 			dbmstype = xmlGetProp (cur, BAD_CAST "dbmstype");
 			gdatype = xmlGetProp (cur, BAD_CAST "gdatype");
-			dtype = utility_find_or_create_data_type (dict, prov, cnc,
-								  dbmstype, gdatype, &dtype_created);
+			dtype = gda_utility_find_or_create_data_type (dict, prov, cnc,
+								  (gchar*)dbmstype, (gchar*)gdatype, &dtype_created);
 			if (dbmstype) xmlFree (dbmstype);
 			if (gdatype) xmlFree (gdatype);
 
@@ -476,7 +656,7 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 
 			if (!param) {
 				param = GDA_PARAMETER (g_object_new (GDA_TYPE_PARAMETER, "dict", dict,
-								     "gda_type", gda_dict_type_get_gda_type (dtype),
+								     "g_type", gda_dict_type_get_g_type (dtype),
 								     NULL));
 				params = g_slist_append (params, param);
 			}
@@ -484,7 +664,7 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 				g_object_unref (dtype);
 			
 			/* set parameter's attributes */
-			utility_parameter_load_attributes (param, cur, sources);
+			gda_utility_parameter_load_attributes (param, cur, sources);
 		}
 	}
 
@@ -511,19 +691,19 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 		xmlChar *prop;;
 		plist = gda_parameter_list_new (params);
 
-		prop = xmlGetProp (xml_spec, "id");
+		prop = xmlGetProp(xml_spec, (xmlChar*)"id");
 		if (prop) {
-			gda_object_set_id (GDA_OBJECT (plist), prop);
+			gda_object_set_id (GDA_OBJECT (plist), (gchar*)prop);
 			xmlFree (prop);
 		}
-		prop = xmlGetProp (xml_spec, "name");
+		prop = xmlGetProp(xml_spec, (xmlChar*)"name");
 		if (prop) {
-			gda_object_set_name (GDA_OBJECT (plist), prop);
+			gda_object_set_name (GDA_OBJECT (plist), (gchar*)prop);
 			xmlFree (prop);
 		}
-		prop = xmlGetProp (xml_spec, "descr");
+		prop = xmlGetProp(xml_spec, (xmlChar*)"descr");
 		if (prop) {
-			gda_object_set_description (GDA_OBJECT (plist), prop);
+			gda_object_set_description (GDA_OBJECT (plist), (gchar*)prop);
 			xmlFree (prop);
 		}
 	}
@@ -572,9 +752,9 @@ gda_parameter_list_get_spec (GdaParameterList *paramlist)
 
 	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), NULL);
 
-	doc = xmlNewDoc ("1.0");
+	doc = xmlNewDoc ((xmlChar*)"1.0");
 	g_return_val_if_fail (doc, NULL);
-	root = xmlNewDocNode (doc, NULL, "data-set-spec", NULL);
+	root = xmlNewDocNode (doc, NULL, (xmlChar*)"data-set-spec", NULL);
 	xmlDocSetRootElement (doc, root);
 
 	/* parameters list */
@@ -586,31 +766,31 @@ gda_parameter_list_get_spec (GdaParameterList *paramlist)
 		gchar *str;
 		const gchar *cstr;
 
-		node = xmlNewTextChild (root, NULL, "parameter", NULL);
+		node = xmlNewTextChild (root, NULL, (xmlChar*)"parameter", NULL);
 		g_object_get (G_OBJECT (param), "string_id", &str, NULL);
 		if (str) {
-			xmlSetProp (node, "id", str);
+			xmlSetProp(node, (xmlChar*)"id", (xmlChar*)str);
 			g_free (str);
 		}
 
 		cstr = gda_object_get_name (GDA_OBJECT (param));
 		if (cstr)
-			xmlSetProp (node, "name", cstr);		
+			xmlSetProp(node, (xmlChar*)"name", (xmlChar*)cstr);		
 		cstr = gda_object_get_description (GDA_OBJECT (param));
 		if (cstr)
-			xmlSetProp (node, "descr", cstr);		
+			xmlSetProp(node, (xmlChar*)"descr", (xmlChar*)cstr);		
 
-		/* dtype = gda_parameter_get_gda_type (param); */
+		/* dtype = gda_parameter_get_g_type (param); */
 /* 		if (dtype) { */
-/* 			xmlSetProp (node, "dbmstype", gda_dict_type_get_sqlname (dtype)); */
-/* 			xmlSetProp (node, "gdatype", gda_type_to_string (gda_dict_type_get_gda_type (dtype))); */
+/* 			xmlSetProp(node, (xmlChar*)"dbmstype", gda_dict_type_get_sqlname (dtype)); */
+/* 			xmlSetProp(node, (xmlChar*)"gdatype", gda_g_type_to_string (gda_dict_type_get_g_type (dtype))); */
 /* 		} */
-		xmlSetProp (node, "gdatype", gda_type_to_string (gda_parameter_get_gda_type (param)));
+		xmlSetProp(node, (xmlChar*)"gdatype", (xmlChar*)gda_g_type_to_string (gda_parameter_get_g_type (param)));
 
-		xmlSetProp (node, "nullok", gda_parameter_get_not_null (param) ? "FALSE" : "TRUE");
+		xmlSetProp(node, (xmlChar*)"nullok", (xmlChar*)(gda_parameter_get_not_null (param) ? "FALSE" : "TRUE"));
 		g_object_get (G_OBJECT (param), "entry_plugin", &str, NULL);
 		if (str) {
-			xmlSetProp (node, "plugin", str);
+			xmlSetProp(node, (xmlChar*)"plugin", (xmlChar*)str);
 			g_free (str);
 		}
 		
@@ -639,6 +819,8 @@ destroyed_param_cb (GdaParameter *param, GdaParameterList *paramlist)
 					      G_CALLBACK (changed_param_cb), paramlist);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (param),
 					      G_CALLBACK (restrict_changed_param_cb), paramlist);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (param),
+					      G_CALLBACK (notify_param_cb), paramlist);
 
 	/* now destroy the GdaParameterListNode and the GdaParameterListSource if necessary */
 	node = gda_parameter_list_find_node_for_param (paramlist, param);
@@ -664,11 +846,34 @@ restrict_changed_param_cb (GdaParameter *param, GdaParameterList *paramlist)
 	compute_public_data (paramlist);
 }
 
+static void
+notify_param_cb (GdaParameter *param, GParamSpec *pspec, GdaParameterList *paramlist)
+{
+	if (!strcmp (pspec->name, "entry-plugin")) {
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'PARAM_PLUGIN_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (paramlist), gda_parameter_list_signals[PARAM_PLUGIN_CHANGED], 0, param);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'PARAM_PLUGIN_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+	if (!strcmp (pspec->name, "use-default-value")) {
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'PARAM_ATTR_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (paramlist), gda_parameter_list_signals[PARAM_ATTR_CHANGED], 0, param);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'PARAM_ATTR_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+}
+
 static void
 changed_param_cb (GdaParameter *param, GdaParameterList *paramlist)
 {
 	/* signal the parameter change */
-	gda_object_changed (GDA_OBJECT (paramlist));
+	gda_object_signal_emit_changed (GDA_OBJECT (paramlist));
 #ifdef GDA_DEBUG_signal
 	g_print (">> 'PARAM_CHANGED' from %s\n", __FUNCTION__);
 #endif
@@ -1014,6 +1219,8 @@ gda_parameter_list_real_add_param (GdaParameterList *paramlist, GdaParameter *pa
 				  G_CALLBACK (changed_param_cb), paramlist);
 		g_signal_connect (G_OBJECT (param), "restrict_changed",
 				  G_CALLBACK (restrict_changed_param_cb), paramlist);
+		g_signal_connect (G_OBJECT (param), "notify",
+				  G_CALLBACK (notify_param_cb), paramlist);
 
 		g_object_ref (G_OBJECT (param));
 		gda_parameter_replace_param_users (param, paramlist->priv->param_repl);
@@ -1043,13 +1250,13 @@ gda_parameter_list_add_param_from_string (GdaParameterList *paramlist, const gch
 	g_return_val_if_fail (paramlist->priv, NULL);
 
 	dict = gda_object_get_dict ((GdaObject*) paramlist);
-	param = g_object_new (GDA_TYPE_PARAMETER, "dict", dict, "gda_type", type, NULL);
+	param = g_object_new (GDA_TYPE_PARAMETER, "dict", dict, "g_type", type, NULL);
 	g_return_val_if_fail (param, NULL);
 
 	if (! gda_parameter_set_value_str (param, str)) {
 		g_object_unref (param);
 		g_warning (_("Could not add parameter of type %s with value '%s'"), 
-			   gda_type_to_string (type), str);
+			   gda_g_type_to_string (type), str);
 		return NULL;
 	}
 
@@ -1083,7 +1290,7 @@ gda_parameter_list_add_param_from_value (GdaParameterList *paramlist, const gcha
 	g_return_val_if_fail (G_IS_VALUE (value), NULL);
 
 	dict = gda_object_get_dict ((GdaObject*) paramlist);
-	param = g_object_new (GDA_TYPE_PARAMETER, "dict", dict, "gda_type", G_VALUE_TYPE (value), NULL);
+	param = g_object_new (GDA_TYPE_PARAMETER, "dict", dict, "g_type", G_VALUE_TYPE (value), NULL);
 	g_return_val_if_fail (param, NULL);
 
 	gda_parameter_set_value (param, value);
@@ -1176,12 +1383,12 @@ gda_parameter_list_is_coherent (GdaParameterList *paramlist, GError **error)
 					     node->source_model, node->source_column);
 				return FALSE;
 			}
-			if (gda_column_get_gda_type (col) != gda_parameter_get_gda_type (node->param)) {
+			if (gda_column_get_g_type (col) != gda_parameter_get_g_type (node->param)) {
 				g_set_error (error, GDA_PARAMETER_LIST_ERROR, 0,
 					     _("GdaParameter is restricted by a column of the wrong type: "
 					       "%s (%s expected)"),
-					     gda_type_to_string (gda_parameter_get_gda_type (node->param)),
-					     gda_type_to_string (gda_column_get_gda_type (col)));
+					     gda_g_type_to_string (gda_parameter_get_g_type (node->param)),
+					     gda_g_type_to_string (gda_column_get_g_type (col)));
 				return FALSE;
 			}
 		}
@@ -1492,7 +1699,7 @@ gda_parameter_list_set_param_default_value (GdaParameterList *paramlist, GdaPara
 
 	if (value && ! gda_value_is_null ((GValue *) value)) {
 		g_return_if_fail (G_VALUE_TYPE ((GValue *) value) ==
-				  gda_parameter_get_gda_type (param));
+				  gda_parameter_get_g_type (param));
 		g_hash_table_insert (paramlist->priv->param_default_values, param, 
 				     gda_value_copy ((GValue *) value));
 	}
@@ -1532,8 +1739,8 @@ gda_parameter_list_set_param_default_alias (GdaParameterList *paramlist, GdaPara
 	if (alias) {
 		g_return_if_fail (alias != param);
 		g_return_if_fail (alias && GDA_IS_PARAMETER (alias));
-		g_return_if_fail (gda_parameter_get_gda_type (param) == 
-				  gda_parameter_get_gda_type (alias));
+		g_return_if_fail (gda_parameter_get_g_type (param) == 
+				  gda_parameter_get_g_type (alias));
 		g_hash_table_insert (paramlist->priv->param_default_aliases, param, alias);
 		g_hash_table_insert (paramlist->priv->aliases_default_param, alias, param);
 		gda_object_connect_destroy (alias, G_CALLBACK (destroyed_alias_cb), paramlist);
@@ -1617,8 +1824,8 @@ gda_parameter_list_dump (GdaParameterList *paramlist, guint offset)
         if (paramlist->priv) {
 		GError *error = NULL;
 		GSList *list;
-                g_print ("%s" D_COL_H1 "GdaParameterList" D_COL_NOR " %p (id=%s)",
-                         str, paramlist, gda_object_get_id (GDA_OBJECT (paramlist)));
+                g_print ("%s" D_COL_H1 "%s" D_COL_NOR " %p (id=%s)",
+                         str, G_OBJECT_TYPE_NAME (paramlist), paramlist, gda_object_get_id (GDA_OBJECT (paramlist)));
 		if (! gda_parameter_list_is_coherent (paramlist, &error)) {
 			g_print (" " D_COL_ERR "not coherent: " D_COL_NOR "%s", error->message);
 			g_error_free (error);
