@@ -1,8 +1,9 @@
 /* GDA Common Library
- * Copyright (C) 1998-2002 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * Authors:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -20,8 +21,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if !defined(__gda_quark_list_h__)
-#  define __gda_quark_list_h__
+#ifndef __GDA_QUARK_LIST_H__
+#define __GDA_QUARK_LIST_H__
 
 #include <glib/gmacros.h>
 #include <glib/gtypes.h>
@@ -33,18 +34,19 @@ typedef struct _GdaQuarkList GdaQuarkList;
 
 #define GDA_TYPE_QUARK_LIST (gda_quark_list_get_type())
 
-GType         gda_quark_list_get_type (void);
-GdaQuarkList *gda_quark_list_new (void);
+GType         gda_quark_list_get_type        (void);
+GdaQuarkList *gda_quark_list_new             (void);
 GdaQuarkList *gda_quark_list_new_from_string (const gchar *string);
-GdaQuarkList *gda_quark_list_copy (GdaQuarkList *qlist);
-void          gda_quark_list_free (GdaQuarkList *qlist);
+GdaQuarkList *gda_quark_list_copy            (GdaQuarkList *qlist);
+void          gda_quark_list_free            (GdaQuarkList *qlist);
 
 void          gda_quark_list_add_from_string (GdaQuarkList *qlist,
 					      const gchar *string,
 					      gboolean cleanup);
-const gchar  *gda_quark_list_find (GdaQuarkList *qlist, const gchar *name);
-void          gda_quark_list_remove (GdaQuarkList *qlist, const gchar *name);
-void          gda_quark_list_clear (GdaQuarkList *qlist);
+const gchar  *gda_quark_list_find            (GdaQuarkList *qlist, const gchar *name);
+void          gda_quark_list_remove          (GdaQuarkList *qlist, const gchar *name);
+void          gda_quark_list_clear           (GdaQuarkList *qlist);
+void          gda_quark_list_foreach         (GdaQuarkList *qlist, GHFunc func, gpointer user_data);
 
 G_END_DECLS
 
