@@ -1,21 +1,21 @@
 /* gda-query-condition.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -66,7 +66,8 @@ static gboolean    gda_query_condition_load_from_xml (GdaXmlStorage *iface, xmlN
 
 /* Renderer interface */
 static void            gda_query_condition_renderer_init      (GdaRendererIface *iface);
-static gchar          *gda_query_condition_render_as_sql   (GdaRenderer *iface, GdaParameterList *context, guint options, GError **error);
+static gchar          *gda_query_condition_render_as_sql   (GdaRenderer *iface, GdaParameterList *context, 
+							    GSList **out_params_used, GdaRendererOptions options, GError **error);
 static gchar          *gda_query_condition_render_as_str   (GdaRenderer *iface, GdaParameterList *context);
 
 /* Referer interface */
@@ -204,11 +205,13 @@ gda_query_condition_class_init (GdaQueryConditionClass * class)
 	object_class->set_property = gda_query_condition_set_property;
 	object_class->get_property = gda_query_condition_get_property;
 	g_object_class_install_property (object_class, PROP_QUERY,
-					 g_param_spec_pointer ("query", "Query to which the condition belongs", NULL, 
+					 g_param_spec_object ("query", "Query to which the condition belongs", NULL, 
+                                                               GDA_TYPE_QUERY,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE |
 								G_PARAM_CONSTRUCT_ONLY)));
 	g_object_class_install_property (object_class, PROP_JOIN,
-					 g_param_spec_pointer ("join", NULL, NULL, 
+					 g_param_spec_object ("join", NULL, NULL, 
+                                                               GDA_TYPE_QUERY_JOIN,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_COND_TYPE,
 					 g_param_spec_int ("cond_type", "Type of condition", NULL,
@@ -248,7 +251,7 @@ gda_query_condition_init (GdaQueryCondition *condition)
  *
  * Returns: the newly created object
  */
-GObject *
+GdaQueryCondition *
 gda_query_condition_new (GdaQuery *query, GdaQueryConditionType type)
 {
 	GObject *obj;
@@ -258,7 +261,7 @@ gda_query_condition_new (GdaQuery *query, GdaQueryConditionType type)
 	obj = g_object_new (GDA_TYPE_QUERY_CONDITION, "dict", gda_object_get_dict (GDA_OBJECT (query)), 
 			    "query", query, "cond_type", type, NULL);
 
-	return obj;
+	return (GdaQueryCondition*) obj;
 }
 
 static void
@@ -281,7 +284,7 @@ ops_ref_lost_cb (GdaObjectRef *refbase, GdaQueryCondition *cond)
  *
  * Returns: the new object
  */
-GObject *
+GdaQueryCondition *
 gda_query_condition_new_copy (GdaQueryCondition *orig, GHashTable *replacements)
 {
 	GObject *obj;
@@ -318,13 +321,14 @@ gda_query_condition_new_copy (GdaQueryCondition *orig, GHashTable *replacements)
 	while (list) {
 		GObject *ccond;
 
-		ccond = gda_query_condition_new_copy (GDA_QUERY_CONDITION (list->data), replacements);
+		ccond = G_OBJECT (gda_query_condition_new_copy (GDA_QUERY_CONDITION (list->data), replacements));
 		gda_query_condition_node_add_child (newcond, GDA_QUERY_CONDITION (ccond), NULL);
 		g_object_unref (ccond);
 		list = g_slist_next (list);
 	}
 
-	return obj;
+	g_object_unref (query);
+	return newcond;
 }
 
 /**
@@ -339,7 +343,7 @@ gda_query_condition_new_copy (GdaQueryCondition *orig, GHashTable *replacements)
  *
  * Returns: a new #GdaQueryCondition, or %NULL if there was an error in @sql_cond
  */
-GObject *
+GdaQueryCondition *
 gda_query_condition_new_from_sql (GdaQuery *query, const gchar *sql_cond, GSList **targets, GError **error)
 {
 	gchar *sql;
@@ -373,7 +377,7 @@ gda_query_condition_new_from_sql (GdaQuery *query, const gchar *sql_cond, GSList
 				     _("Error parsing '%s'"), sql_cond);
 	g_free (sql);
 
-	return G_OBJECT (newcond);
+	return newcond;
 }
 
 static void
@@ -459,7 +463,7 @@ destroyed_child_cb (GdaQueryCondition *child, GdaQueryCondition *cond)
 	g_object_unref (G_OBJECT (child));
 	cond->priv->cond_children = g_slist_remove (cond->priv->cond_children, child);
 	if (! cond->priv->internal_transaction)
-		gda_object_changed (GDA_OBJECT (cond));
+		gda_object_signal_emit_changed (GDA_OBJECT (cond));
 }
 
 
@@ -489,7 +493,6 @@ gda_query_condition_set_property (GObject *object,
 				  GParamSpec *pspec)
 {
 	GdaQueryCondition *condition;
-	gpointer ptr;
 	guint id;
 	gint i;
 	GdaDict *dict;
@@ -497,8 +500,8 @@ gda_query_condition_set_property (GObject *object,
 	condition = GDA_QUERY_CONDITION (object);
 	if (condition->priv) {
 		switch (param_id) {
-		case PROP_QUERY:
-			ptr = g_value_get_pointer (value);
+		case PROP_QUERY: {
+			GdaQuery* ptr = g_value_get_object (value);
                         g_return_if_fail (ptr && GDA_IS_QUERY (ptr));
 
                         if (condition->priv->query) {
@@ -515,7 +518,7 @@ gda_query_condition_set_property (GObject *object,
 			gda_object_connect_destroy (ptr, G_CALLBACK (destroyed_object_cb), condition);
 			gda_query_declare_condition (condition->priv->query, condition);
 
-			dict = gda_object_get_dict (ptr);
+			dict = gda_object_get_dict (GDA_OBJECT(ptr));
 			for (i=0; i<3; i++) {
 				condition->priv->ops[i] = GDA_OBJECT_REF (gda_object_ref_new (dict));
 				g_signal_connect (G_OBJECT (condition->priv->ops[i]), "ref_lost",
@@ -525,8 +528,9 @@ gda_query_condition_set_property (GObject *object,
 			g_object_get (G_OBJECT (ptr), "cond_serial", &id, NULL);
 			gda_query_object_set_int_id (GDA_QUERY_OBJECT (object), id);			
 			break;
-		case PROP_JOIN:
-			ptr = g_value_get_pointer (value);
+                }
+		case PROP_JOIN: {
+			GdaQueryJoin* ptr = GDA_QUERY_JOIN (g_value_get_object (value));
 			if (ptr) {
 				g_return_if_fail (GDA_IS_QUERY_JOIN (ptr));
 				g_return_if_fail (gda_query_join_get_query (GDA_QUERY_JOIN (ptr)) == condition->priv->query);
@@ -547,12 +551,13 @@ gda_query_condition_set_property (GObject *object,
 				gda_object_connect_destroy (ptr, G_CALLBACK (destroyed_object_cb), condition);
 			}
 			break;
+                }
 		case PROP_COND_TYPE:
 			if (condition->priv->type == g_value_get_int (value))
 				return;
 
 			condition->priv->type = g_value_get_int (value);
-			gda_object_changed (GDA_OBJECT (condition));
+			gda_object_signal_emit_changed (GDA_OBJECT (condition));
 			break;
 		}
 	}
@@ -570,10 +575,10 @@ gda_query_condition_get_property (GObject *object,
         if (condition->priv) {
                 switch (param_id) {
                 case PROP_QUERY:
-			g_value_set_pointer (value, condition->priv->query);
+			g_value_set_object (value, G_OBJECT (condition->priv->query));
                         break;
 		case PROP_JOIN:
-			g_value_set_pointer (value, condition->priv->join);
+			g_value_set_object (value, G_OBJECT (condition->priv->join));
                         break;
 		case PROP_COND_TYPE:
 			g_value_set_int (value, condition->priv->type);
@@ -624,7 +629,7 @@ gda_query_condition_set_cond_type (GdaQueryCondition *condition, GdaQueryConditi
 
 	condition->priv->type = type;
 	if (! condition->priv->internal_transaction)
-		gda_object_changed (GDA_OBJECT (condition));
+		gda_object_signal_emit_changed (GDA_OBJECT (condition));
 }
 
 /**
@@ -811,7 +816,7 @@ gda_query_condition_node_add_child_pos (GdaQueryCondition *condition, GdaQueryCo
 			  G_CALLBACK (child_cond_changed_cb), condition);
 
 	if (! condition->priv->internal_transaction)
-		gda_object_changed (GDA_OBJECT (condition));
+		gda_object_signal_emit_changed (GDA_OBJECT (condition));
 
 	return TRUE;
 }
@@ -821,7 +826,7 @@ static void
 child_cond_changed_cb (GdaQueryCondition *child, GdaQueryCondition *cond)
 {
 	if (! cond->priv->internal_transaction)
-		gda_object_changed (GDA_OBJECT (cond));
+		gda_object_signal_emit_changed (GDA_OBJECT (cond));
 }
 
 /**
@@ -866,6 +871,8 @@ gda_query_condition_leaf_set_operator (GdaQueryCondition *condition, GdaQueryCon
 	g_object_get (G_OBJECT (field), "query", &query2, NULL);
 	g_return_if_fail (query1);
 	g_return_if_fail (query1 == query2);
+	g_object_unref (query1);
+	g_object_unref (query2);
 
 	gda_object_ref_set_ref_object_type (condition->priv->ops[op], GDA_OBJECT (field), GDA_TYPE_QUERY_FIELD);
 }
@@ -1366,32 +1373,32 @@ gda_query_condition_save_to_xml (GdaXmlStorage *iface, GError **error)
 
         cond = GDA_QUERY_CONDITION (iface);
 
-        node = xmlNewNode (NULL, "gda_query_cond");
+        node = xmlNewNode (NULL, (xmlChar*)"gda_query_cond");
 
         str = gda_xml_storage_get_xml_id (iface);
-        xmlSetProp (node, "id", str);
+        xmlSetProp(node, (xmlChar*)"id", (xmlChar*)str);
         g_free (str);
 
-        xmlSetProp (node, "type", condition_type_to_str (cond->priv->type));
+        xmlSetProp(node, (xmlChar*)"type", (xmlChar*)condition_type_to_str (cond->priv->type));
 
 	base = gda_object_ref_get_ref_object (cond->priv->ops[GDA_QUERY_CONDITION_OP_LEFT]);
 	if (base) {
 		str = gda_xml_storage_get_xml_id (GDA_XML_STORAGE (base));
-		xmlSetProp (node, "l_op", str);
+		xmlSetProp(node, (xmlChar*)"l_op", (xmlChar*)str);
 		g_free (str);
 	}
 
 	base = gda_object_ref_get_ref_object (cond->priv->ops[GDA_QUERY_CONDITION_OP_RIGHT]);
 	if (base) {
 		str = gda_xml_storage_get_xml_id (GDA_XML_STORAGE (base));
-		xmlSetProp (node, "r_op", str);
+		xmlSetProp(node, (xmlChar*)"r_op", (xmlChar*)str);
 		g_free (str);
 	}
 
 	base = gda_object_ref_get_ref_object (cond->priv->ops[GDA_QUERY_CONDITION_OP_RIGHT2]);
 	if (base) {
 		str = gda_xml_storage_get_xml_id (GDA_XML_STORAGE (base));
-		xmlSetProp (node, "r_op2", str);
+		xmlSetProp(node, (xmlChar*)"r_op2", (xmlChar*)str);
 		g_free (str);
 	}
 
@@ -1425,7 +1432,7 @@ gda_query_condition_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
 
 	cond = GDA_QUERY_CONDITION (iface);
 
-	if (strcmp (node->name, "gda_query_cond")) {
+	if (strcmp ((gchar*)node->name, "gda_query_cond")) {
                 g_set_error (error,
                              GDA_QUERY_CONDITION_ERROR,
                              GDA_QUERY_CONDITION_XML_LOAD_ERROR,
@@ -1433,7 +1440,7 @@ gda_query_condition_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
                 return FALSE;
         }
 
-	prop = xmlGetProp (node, "id");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"id");
         if (prop) {
                 gchar *ptr, *tok;
                 ptr = strtok_r (prop, ":", &tok);
@@ -1450,7 +1457,7 @@ gda_query_condition_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
                 g_free (prop);
         }
 
-	prop = xmlGetProp (node, "type");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"type");
         if (prop) {
 		cond->priv->type = condition_str_to_type (prop);
 		if (cond->priv->type == GDA_QUERY_CONDITION_TYPE_UNKNOWN) {
@@ -1463,7 +1470,7 @@ gda_query_condition_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
                 g_free (prop);
         }
 
-	prop = xmlGetProp (node, "l_op");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"l_op");
 	if (prop) {
 		gda_object_ref_set_ref_name (cond->priv->ops[GDA_QUERY_CONDITION_OP_LEFT], GDA_TYPE_QUERY_FIELD,
 					  REFERENCE_BY_XML_ID, prop);
@@ -1471,14 +1478,14 @@ gda_query_condition_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
 	}
 
 
-	prop = xmlGetProp (node, "r_op");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"r_op");
 	if (prop) {
 		gda_object_ref_set_ref_name (cond->priv->ops[GDA_QUERY_CONDITION_OP_RIGHT], GDA_TYPE_QUERY_FIELD,
 					  REFERENCE_BY_XML_ID, prop);
 		g_free (prop);
 	}
 
-	prop = xmlGetProp (node, "r_op2");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"r_op2");
 	if (prop) {
 		gda_object_ref_set_ref_name (cond->priv->ops[GDA_QUERY_CONDITION_OP_RIGHT2], GDA_TYPE_QUERY_FIELD,
 					  REFERENCE_BY_XML_ID, prop);
@@ -1488,10 +1495,10 @@ gda_query_condition_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
 	/* children nodes */
 	children = node->children;
 	while (children) {
-		if (!strcmp (children->name, "gda_query_cond")) {
+		if (!strcmp ((gchar*)children->name, "gda_query_cond")) {
 			GdaQueryCondition *scond;
 
-			scond = GDA_QUERY_CONDITION (gda_query_condition_new (cond->priv->query, GDA_QUERY_CONDITION_NODE_AND));
+			scond = gda_query_condition_new (cond->priv->query, GDA_QUERY_CONDITION_NODE_AND);
 			if (gda_xml_storage_load_from_xml (GDA_XML_STORAGE (scond), children, error)) {
 				gda_query_condition_node_add_child (cond, scond, NULL);
 				g_object_unref (G_OBJECT (scond));
@@ -1520,7 +1527,8 @@ gda_query_condition_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
  */
 
 static gchar *
-gda_query_condition_render_as_sql (GdaRenderer *iface, GdaParameterList *context, guint options, GError **error)
+gda_query_condition_render_as_sql (GdaRenderer *iface, GdaParameterList *context, 
+				   GSList **out_params_used, GdaRendererOptions options, GError **error)
 {
         gchar *retval = NULL, *str;
 	GString *string;
@@ -1690,7 +1698,7 @@ gda_query_condition_render_as_sql (GdaRenderer *iface, GdaParameterList *context
 						g_string_append (string, "(");
 
 					str = gda_renderer_render_as_sql (GDA_RENDERER (list->data), context, 
-									 options, error);
+									  out_params_used, options, error);
 					if (!str) {
 						g_string_free (string, TRUE);
 						return NULL;
@@ -1707,7 +1715,7 @@ gda_query_condition_render_as_sql (GdaRenderer *iface, GdaParameterList *context
 			else { /* type = NOT */
 				g_string_append_printf (string, "%s", link);
 				str = gda_renderer_render_as_sql (GDA_RENDERER (cond->priv->cond_children->data), 
-								 context, options, error);
+								  context, out_params_used, options, error);
 				if (!str) {
 					g_string_free (string, TRUE);
 					return NULL;
@@ -1720,7 +1728,7 @@ gda_query_condition_render_as_sql (GdaRenderer *iface, GdaParameterList *context
 		}
 		else {
 			str = gda_renderer_render_as_sql (GDA_RENDERER (ops[GDA_QUERY_CONDITION_OP_LEFT]), context, 
-							 options, error);
+							  out_params_used, options, error);
 			if (!str) {
 				g_string_free (string, TRUE);
 				return NULL;
@@ -1729,7 +1737,7 @@ gda_query_condition_render_as_sql (GdaRenderer *iface, GdaParameterList *context
 			g_free (str);
 			g_string_append_printf (string, "%s", link);
 			str = gda_renderer_render_as_sql (GDA_RENDERER (ops[GDA_QUERY_CONDITION_OP_RIGHT]), context, 
-							 options, error);
+							  out_params_used, options, error);
 			if (!str) {
 				g_string_free (string, TRUE);
 				return NULL;
@@ -1741,7 +1749,7 @@ gda_query_condition_render_as_sql (GdaRenderer *iface, GdaParameterList *context
 
 	if (cond->priv->type == GDA_QUERY_CONDITION_LEAF_BETWEEN) {
 		str = gda_renderer_render_as_sql (GDA_RENDERER (ops[GDA_QUERY_CONDITION_OP_RIGHT2]), context, 
-						 options, error);
+						  out_params_used, options, error);
 		if (!str) {
 			g_string_free (string, TRUE);
 			return NULL;
@@ -1764,7 +1772,7 @@ gda_query_condition_render_as_str (GdaRenderer *iface, GdaParameterList *context
         g_return_val_if_fail (iface && GDA_IS_QUERY_CONDITION (iface), NULL);
         g_return_val_if_fail (GDA_QUERY_CONDITION (iface)->priv, NULL);
 
-        str = gda_query_condition_render_as_sql (iface, context, 0, NULL);
+        str = gda_query_condition_render_as_sql (iface, context, NULL, 0, NULL);
         if (!str)
                 str = g_strdup ("???");
         return str;
