@@ -1,21 +1,21 @@
 /* gda-query-target.c
  *
- * Copyright (C) 2003 - 2006 Vivien Malerba
+ * Copyright (C) 2003 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -63,7 +63,8 @@ static void        gda_query_target_replace_refs        (GdaReferer *iface, GHas
 
 /* Renderer interface */
 static void        gda_query_target_renderer_init   (GdaRendererIface *iface);
-static gchar      *gda_query_target_render_as_sql   (GdaRenderer *iface, GdaParameterList *context, guint options, GError **error);
+static gchar      *gda_query_target_render_as_sql   (GdaRenderer *iface, GdaParameterList *context, 
+						     GSList **out_params_used, GdaRendererOptions options, GError **error);
 static gchar      *gda_query_target_render_as_str   (GdaRenderer *iface, GdaParameterList *context);
 
 /* Alias interface */
@@ -232,7 +233,6 @@ gda_query_target_init (GdaQueryTarget *target)
  * gda_query_target_new
  * @query: a #GdaQuery object
  * @table: the name of the table to reference
- * @as: optional alias
  *
  * Creates a new #GdaQueryTarget object, specifying the name of the table to reference.
  *
@@ -736,10 +736,10 @@ gda_query_target_save_to_xml (GdaXmlStorage *iface, GError **error)
 
 	target = GDA_QUERY_TARGET (iface);
 
-	node = xmlNewNode (NULL, "gda_query_target");
+	node = xmlNewNode (NULL, (xmlChar*)"gda_query_target");
 	
 	str = gda_xml_storage_get_xml_id (iface);
-	xmlSetProp (node, "id", str);
+	xmlSetProp(node, (xmlChar*)"id", (xmlChar*)str);
 	g_free (str);
 	
 	if (target->priv->entity_ref) {
@@ -749,7 +749,7 @@ gda_query_target_save_to_xml (GdaXmlStorage *iface, GError **error)
 		if (base) {
 			str = gda_xml_storage_get_xml_id (GDA_XML_STORAGE (base));
 			if (str) {
-				xmlSetProp (node, "entity_ref", str);
+				xmlSetProp(node, (xmlChar*)"entity_ref", (xmlChar*)str);
 				g_free (str);
 			}
 			else {
@@ -763,7 +763,7 @@ gda_query_target_save_to_xml (GdaXmlStorage *iface, GError **error)
 		else {
 			str = (gchar *) gda_object_ref_get_ref_name (target->priv->entity_ref, NULL, NULL);
 			if (str) 
-				xmlSetProp (node, "table_name", str);
+				xmlSetProp(node, (xmlChar*)"table_name", (xmlChar*)str);
 			else {
 				g_set_error (error, GDA_QUERY_TARGET_ERROR,
 					     GDA_QUERY_TARGET_XML_SAVE_ERROR,
@@ -790,7 +790,7 @@ gda_query_target_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError **
 	g_return_val_if_fail (node, FALSE);
 
 	target = GDA_QUERY_TARGET (iface);
-	if (strcmp (node->name, "gda_query_target")) {
+	if (strcmp ((gchar*)node->name, "gda_query_target")) {
 		g_set_error (error,
 			     GDA_QUERY_TARGET_ERROR,
 			     GDA_QUERY_TARGET_XML_LOAD_ERROR,
@@ -798,7 +798,7 @@ gda_query_target_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError **
 		return FALSE;
 	}
 
-	str = xmlGetProp (node, "id");
+	str = (gchar*)xmlGetProp(node, (xmlChar*)"id");
 	if (str) {
 		gchar *tok, *ptr;
 
@@ -821,7 +821,7 @@ gda_query_target_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError **
 		g_free (str);
 	}
 
-	str = xmlGetProp (node, "entity_ref");
+	str = (char*)xmlGetProp(node, (xmlChar*)"entity_ref");
 	if (str) {
 		GType target_ref;
 		const gchar *name;
@@ -847,7 +847,7 @@ gda_query_target_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError **
 		g_free (str);
 	}
 	else {
-		str = xmlGetProp (node, "table_name");
+		str = (gchar*)xmlGetProp(node, (xmlChar*)"table_name");
 		if (str) {
 			g_assert (target->priv->entity_ref);
 			gda_object_ref_set_ref_name (target->priv->entity_ref, GDA_TYPE_DICT_TABLE, 
@@ -875,7 +875,8 @@ gda_query_target_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError **
  */
 
 static gchar *
-gda_query_target_render_as_sql (GdaRenderer *iface, GdaParameterList *context, guint options, GError **error)
+gda_query_target_render_as_sql (GdaRenderer *iface, GdaParameterList *context, 
+				GSList **out_params_used, GdaRendererOptions options, GError **error)
 {
 	gchar *str;
 	GString *string = NULL;
@@ -915,7 +916,7 @@ gda_query_target_render_as_sql (GdaRenderer *iface, GdaParameterList *context, g
 		
 		if (GDA_IS_QUERY (entity)) {
 			string = g_string_new ("(");
-			str = gda_renderer_render_as_sql (GDA_RENDERER (entity), context, options, error);
+			str = gda_renderer_render_as_sql (GDA_RENDERER (entity), context, out_params_used, options, error);
 			if (str) {
 				g_string_append (string, str);
 				g_free (str);
