@@ -251,7 +251,7 @@ gda_server_operation_get_type (void)
 {
 	static GType type = 0;
 
-	if (G_UNLIKELY (type == 0)) {
+	if (!type) {
 		static const GTypeInfo info = {
 			sizeof (GdaServerOperationClass),
 			(GBaseInitFunc) NULL,
@@ -587,7 +587,7 @@ gda_server_operation_set_property (GObject *object,
 					old_dtd = doc->intSubset;
 					doc->intSubset = gda_server_op_dtd;
 				}
-#ifndef G_OS_WIN32
+
 				if (doc->intSubset && !xmlValidateDocument (validc, doc)) {
 					gchar *str;
 					
@@ -609,7 +609,6 @@ gda_server_operation_set_property (GObject *object,
 					xmlDoValidityCheckingDefaultValue = xmlcheck;
 					return;
 				}
-#endif
 				
 				xmlDoValidityCheckingDefaultValue = xmlcheck;
 				g_free (validc);
@@ -750,17 +749,25 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 
 		/* don't care about entries for the wrong locale */
 		this_lang = xmlGetProp(node, (xmlChar*)"lang");
-		if (this_lang && strncmp ((gchar*)this_lang, lang, strlen ((gchar*)this_lang))) {
-			g_free (this_lang);
+		if (this_lang) {
+			if (strncmp ((gchar*)this_lang, lang, strlen ((gchar*)this_lang))) {
+				xmlFree (this_lang);
+				node = node->next;
+				continue;
+			}
+
+			xmlFree(this_lang);
+		}
+
+		id = xmlGetProp (node, BAD_CAST "id");
+		if (!id) {
 			node = node->next;
 			continue;
 		}
 
-		id = xmlGetProp (node, BAD_CAST "id");
 		complete_path = g_strdup_printf ("%s/%s", root ? root : "", id);
 		path_name = g_strdup ((gchar*)id);
-		if (id)
-			xmlFree (id);
+		xmlFree (id);
 
 		old_opnode = node_find (op, complete_path);
 		if (old_opnode) {
