@@ -89,7 +89,6 @@ gda_server_provider_init (GdaServerProvider *provider,
 	g_return_if_fail (GDA_IS_SERVER_PROVIDER (provider));
 
 	provider->priv = g_new0 (GdaServerProviderPrivate, 1);
-	provider->priv->connections = NULL;
 	provider->priv->data_handlers = g_hash_table_new_full ((GHashFunc) gda_server_provider_handler_info_hash_func,
 							       (GEqualFunc) gda_server_provider_handler_info_equal_func,
 							       (GDestroyNotify) gda_server_provider_handler_info_free,
@@ -105,7 +104,6 @@ gda_server_provider_finalize (GObject *object)
 
 	/* free memory */
 	if (provider->priv) {
-		g_list_free (provider->priv->connections);
 		g_hash_table_destroy (provider->priv->data_handlers);
 		
 		g_free (provider->priv);
@@ -233,15 +231,6 @@ gda_server_provider_open_connection (GdaServerProvider *provider,
 	}
 
 	retcode = CLASS (provider)->open_connection (provider, cnc, params, username, password);
-	if (retcode) {
-		provider->priv->connections = g_list_append (
-			provider->priv->connections, cnc);
-	}
-	else {
-		/* unref the object if we've got no connections */
-		if (!provider->priv->connections)
-			g_object_unref (G_OBJECT (provider));
-	}
 
 	return retcode;
 }
@@ -267,10 +256,6 @@ gda_server_provider_close_connection (GdaServerProvider *provider, GdaConnection
 	else
 		retcode = TRUE;
 
-	provider->priv->connections = g_list_remove (provider->priv->connections, cnc);
-	if (!provider->priv->connections)
-		g_object_unref (G_OBJECT (provider));
-
 	return retcode;
 }
 
@@ -723,6 +708,10 @@ gda_server_provider_execute_query (GdaServerProvider *provider,
  * A connection must be specified, and, optionally, a result set. If not NULL,
  * the provider should try to get the last insert ID for the given result set.
  *
+ * Beware however that the interpretation and usage of this value depends on the
+ * DBMS type being used, see the <link linkend="limitations">limitations</link> 
+ * of each DBMS for more information.
+ *
  * Returns: a string representing the ID of the last inserted row, or NULL
  * if an error occurred or no row has been inserted. It is the caller's
  * reponsibility to free the returned string.
