@@ -1,9 +1,10 @@
 /* GDA FireBird Provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation
+ * Copyright (C) 1998 - 2007 The GNOME Foundation
  *
  * AUTHORS:
  *         Albi Jeronimo <jeronimoalbi@yahoo.com.ar>
  *         Rodrigo Moya <rodrigo@gnome-db.org>
+ *         Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,7 +29,8 @@
 #include <string.h>
 #include "gda-firebird-provider.h"
 #include "gda-firebird-recordset.h"
-#include "gda-firebird-blob.h"
+
+#include <libgda/sql-delimiter/gda-sql-delimiter.h>
 
 static void 		gda_firebird_provider_class_init (GdaFirebirdProviderClass *klass);
 static void 		gda_firebird_provider_init (GdaFirebirdProvider *provider,
@@ -50,25 +52,20 @@ static const gchar 	*gda_firebird_provider_get_database (GdaServerProvider *prov
 static gboolean		gda_firebird_provider_change_database (GdaServerProvider *provider,
 							       GdaConnection *cnc,
 							       const gchar *name);
-static gboolean 	gda_firebird_provider_create_database_cnc (GdaServerProvider *provider,
-					  		           GdaConnection *cnc,
-							           const gchar *name);
-static gboolean 	gda_firebird_provider_drop_database_cnc (GdaServerProvider *provider,
-								 GdaConnection *cnc,
-								 const gchar *name);
 static GList 		*gda_firebird_provider_execute_command (GdaServerProvider *provider,
 								GdaConnection *cnc,
 								GdaCommand *cmd,
 								GdaParameterList *params);
 static gboolean 	gda_firebird_provider_begin_transaction (GdaServerProvider *provider,
 								 GdaConnection *cnc,
-								 GdaTransaction *xaction);
+								 const gchar *name, GdaTransactionIsolation level,
+								 GError **error);
 static gboolean 	gda_firebird_provider_commit_transaction (GdaServerProvider *provider,
 								  GdaConnection *cnc,
-								  GdaTransaction *xaction);
+								  const gchar *name, GError **error);
 static gboolean 	gda_firebird_provider_rollback_transaction (GdaServerProvider *provider,
 								    GdaConnection *cnc,
-								    GdaTransaction *xaction);
+								    const gchar *name, GError **error);
 static gboolean 	gda_firebird_provider_supports (GdaServerProvider *provider,
 							GdaConnection *cnc,
 							GdaConnectionFeature feature);
@@ -77,8 +74,6 @@ static GdaDataModel	*gda_firebird_provider_get_schema (GdaServerProvider *provid
 							   GdaConnectionSchema schema,
 							   GdaParameterList *params);
 
-static GdaBlob          *gda_firebird_provider_create_blob (GdaServerProvider *provider,
-							    GdaConnection *cnc);
 
 static GObjectClass *parent_class = NULL;
 
@@ -97,7 +92,7 @@ gda_firebird_provider_class_init (GdaFirebirdProviderClass *klass)
 	provider_class->get_version = gda_firebird_provider_get_version;
 	provider_class->get_server_version = gda_firebird_provider_get_server_version;
 	provider_class->get_info = NULL;
-	provider_class->supports = gda_firebird_provider_supports;
+	provider_class->supports_feature = gda_firebird_provider_supports;
 	provider_class->get_schema = gda_firebird_provider_get_schema;
 
 	provider_class->get_data_handler = NULL;
@@ -105,30 +100,25 @@ gda_firebird_provider_class_init (GdaFirebirdProviderClass *klass)
 	provider_class->get_def_dbms_type = NULL;
 
 	provider_class->open_connection = gda_firebird_provider_open_connection;
-	provider_class->reset_connection = NULL;
 	provider_class->close_connection = gda_firebird_provider_close_connection;
 	provider_class->get_database = gda_firebird_provider_get_database;
 	provider_class->change_database = gda_firebird_provider_change_database;
 
-	provider_class->get_specs = NULL;
-	provider_class->perform_action_params = NULL;
-
-	provider_class->create_database_cnc = gda_firebird_provider_create_database_cnc;
-	provider_class->drop_database_cnc = gda_firebird_provider_drop_database_cnc;
-	provider_class->create_table = NULL;
-	provider_class->drop_table = NULL;
-	provider_class->create_index = NULL;
-	provider_class->drop_index = NULL;
+	provider_class->supports_operation = NULL;
+        provider_class->create_operation = NULL;
+        provider_class->render_operation = NULL;
+        provider_class->perform_operation = NULL;
 
 	provider_class->execute_command = gda_firebird_provider_execute_command;
+	provider_class->execute_query = NULL;
 	provider_class->get_last_insert_id = NULL;
 
 	provider_class->begin_transaction = gda_firebird_provider_begin_transaction;
 	provider_class->commit_transaction = gda_firebird_provider_commit_transaction;
 	provider_class->rollback_transaction = gda_firebird_provider_rollback_transaction;
-	
-	provider_class->create_blob = gda_firebird_provider_create_blob;
-	provider_class->fetch_blob = NULL;
+	provider_class->add_savepoint = NULL;
+	provider_class->rollback_savepoint = NULL;
+	provider_class->delete_savepoint = NULL;
 }
 
 static void
@@ -233,20 +223,20 @@ fb_get_types (GdaConnection *cnc,
 		const gchar *name;
 		const gchar *owner;
 		const gchar *comments;
-		GdaValueType type;
+		GType type;
 	} types[] = {
-		{ "blob", "", "Binary large object (blob)", GDA_VALUE_TYPE_BINARY },
-		{ "char", "", "Fixed length string", GDA_VALUE_TYPE_STRING },
-		{ "varchar", "", "Variable length string", GDA_VALUE_TYPE_STRING },
-		{ "date", "", "Date", GDA_VALUE_TYPE_DATE },
-		{ "time", "", "Time", GDA_VALUE_TYPE_TIME },
-		{ "timestamp", "", "Time stamp", GDA_VALUE_TYPE_TIMESTAMP },
-		{ "smallint", "", "Signed short integer", GDA_VALUE_TYPE_SMALLINT },
-		{ "integer", "", "Signed long integer (longword)", GDA_VALUE_TYPE_INTEGER },
-		{ "decimal", "", "Decimal number", GDA_VALUE_TYPE_NUMERIC },
-		{ "numeric", "", "Decimal number", GDA_VALUE_TYPE_NUMERIC },
-		{ "float", "", "Single precision number", GDA_VALUE_TYPE_SINGLE },
-		{ "double precision", "", "Double precision number", GDA_VALUE_TYPE_DOUBLE }
+		{ "blob", "", "Binary large object (blob)", GDA_TYPE_BINARY },
+		{ "char", "", "Fixed length string", G_TYPE_STRING },
+		{ "varchar", "", "Variable length string", G_TYPE_STRING },
+		{ "date", "", "Date", G_TYPE_DATE },
+		{ "time", "", "Time", GDA_TYPE_TIME },
+		{ "timestamp", "", "Time stamp", GDA_TYPE_TIMESTAMP },
+		{ "smallint", "", "Signed short integer", GDA_TYPE_SHORT },
+		{ "integer", "", "Signed long integer (longword)", G_TYPE_INT },
+		{ "decimal", "", "Decimal number", GDA_TYPE_NUMERIC },
+		{ "numeric", "", "Decimal number", GDA_TYPE_NUMERIC },
+		{ "float", "", "Single precision number", G_TYPE_FLOAT },
+		{ "double precision", "", "Double precision number", G_TYPE_DOUBLE }
 	};
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -261,11 +251,19 @@ fb_get_types (GdaConnection *cnc,
 	/* Fill the recordset with each of the values from the array defined above */
 	for (i = 0; i < (sizeof (types) / sizeof (types[0])); i++) {
 		value_list = NULL;
+		GValue *tmpval;
 		
-		value_list = g_list_append (value_list, gda_value_new_string (types[i].name));
-		value_list = g_list_append (value_list, gda_value_new_string (types[i].owner));
-		value_list = g_list_append (value_list, gda_value_new_string (types[i].comments));
-		value_list = g_list_append (value_list, gda_value_new_gdatype (types[i].type));
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), types[i].name);
+		value_list = g_list_append (value_list, tmpval);
+
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), types[i].owner);
+		value_list = g_list_append (value_list, tmpval);
+
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), types[i].comments);
+		value_list = g_list_append (value_list, tmpval);
+
+		g_value_set_ulong (tmpval = gda_value_new (G_TYPE_ULONG), types[i].type);
+		value_list = g_list_append (value_list, tmpval);
 
 		/* Add values to row */
 		gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list, NULL);
@@ -294,11 +292,10 @@ fb_get_tables (GdaConnection *cnc,
 	       GdaParameterList *params,
 	       gboolean show_views)
 {
-	GList *reclist, *value_list;
+	GList *value_list;
 	GdaDataModel *recset = NULL;
-	GdaTransaction *transaction;
 	GdaCommand *command;
-	GdaValue *value;
+	GValue *value;
 	GdaRow *row;
 	GdaParameter *par = NULL;
 	gchar *sql, *sql_cond;
@@ -307,16 +304,16 @@ fb_get_tables (GdaConnection *cnc,
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
-	transaction = gda_transaction_new ("temp_transaction");
-	if (gda_connection_begin_transaction (cnc, transaction)) {
-
+	if (gda_connection_begin_transaction (cnc, "temp_transaction", GDA_TRANSACTION_ISOLATION_UNKNOWN, NULL)) {
+		GdaDataModel *recmodel;
+		
 		/* Find parameters */
 		if (params)
 			par = gda_parameter_list_find (params, "systables");
 
 		/* Initialize parameter values */
 		if (par)
-			systables = gda_value_get_boolean ((GdaValue *) gda_parameter_get_value (par));
+			systables = g_value_get_boolean ((GValue *) gda_parameter_get_value (par));
 	
 		/* Evaluate statement conditions */	
 		if (show_views) {
@@ -337,90 +334,83 @@ fb_get_tables (GdaConnection *cnc,
 		
 		/* Execute statement */
 		command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
-		gda_command_set_transaction (command, transaction);		
-		reclist = gda_connection_execute_command (cnc, command, NULL, NULL);
-
-		if (reclist) {
+		recmodel = gda_connection_execute_select_command (cnc, command, NULL, NULL);
+		if (recmodel) {
 			recset = gda_data_model_array_new (4);
-			gda_data_model_set_column_title (recset, 0, _("Table"));
-			gda_data_model_set_column_title (recset, 1, _("Owner"));
-			gda_data_model_set_column_title (recset, 2, _("Description"));
-			gda_data_model_set_column_title (recset, 3, _("Definition"));			
-			for (i = 0; i < gda_data_model_get_n_rows (GDA_DATA_MODEL (reclist->data)); i++) {
+			gda_server_provider_init_schema_model (recset, GDA_CONNECTION_SCHEMA_TABLES);
+			for (i = 0; i < gda_data_model_get_n_rows (recmodel); i++) {
 				value_list = NULL;
+				GValue *tmpval;
 
 				/* Get name of table */
-				row = (GdaRow *) gda_data_model_get_row (GDA_DATA_MODEL (reclist->data), i);
+				row = (GdaRow *) gda_data_model_get_row (recmodel, i);
 				value = gda_row_get_value (row, 0);
-				value_list = g_list_append (
-						value_list,
-						gda_value_new_string ((gchar *) gda_value_get_string (value)));
+				value_list = g_list_append (value_list, gda_value_copy (value));
 
 				/* Get owner of table */
 				value = gda_row_get_value (row, 1);
-				value_list = g_list_append (
-						value_list,
-						gda_value_new_string ((gchar *) gda_value_get_string (value)));
+				value_list = g_list_append (value_list, gda_value_copy (value));
 				
 				/* FIXME: Set correct values */
-				value_list = g_list_append (value_list, gda_value_new_string (""));
-				value_list = g_list_append (value_list, gda_value_new_string (""));
+				g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "");
+				value_list = g_list_append (value_list, tmpval);
+
+				g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "");
+				value_list = g_list_append (value_list, tmpval);
 
 				gda_data_model_append_values (recset, value_list, NULL);
 				g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
 				g_list_free (value_list);
 			}
 
-			g_list_free (reclist);
+			g_object_unref (recmodel);
 		}
 
 		g_free (sql);
-		gda_connection_rollback_transaction (cnc, transaction);
+		gda_connection_rollback_transaction (cnc, "temp_transaction", NULL);
 		gda_command_free (command);
 	}
 	
-	g_object_unref (transaction);
-	
 	return recset;
 }
 
 
 /*
- *  fb_type_name_to_gda_type
+ *  fb_type_name_to_g_type
  *
  *  Function convert a string containing the name of Firebird's
  *  data type to it's corresponding gda data type.
  *
- *  Returns: The GdaValueType corresponding to @name Firebird's
+ *  Returns: The GType corresponding to @name Firebird's
  *           type
  */
-static GdaValueType
-fb_type_name_to_gda_type (const gchar *name)
+static GType
+fb_type_name_to_g_type (const gchar *name)
 {
 	if (!strcmp (name, "CHAR") || !strcmp (name, "VARCHAR"))
-		return GDA_VALUE_TYPE_STRING;
+		return G_TYPE_STRING;
 	else if (!strcmp (name, "SMALLINT"))
-		return GDA_VALUE_TYPE_SMALLINT;
+		return GDA_TYPE_SHORT;
 	else if (!strcmp (name, "NUMERIC") || !strcmp (name, "DECIMAL"))
-		return GDA_VALUE_TYPE_NUMERIC;
+		return GDA_TYPE_NUMERIC;
 	else if (!strcmp (name, "INTEGER"))
-		return GDA_VALUE_TYPE_INTEGER;
+		return G_TYPE_INT;
 	else if (!strcmp (name, "BLOB"))
-		return GDA_VALUE_TYPE_BLOB;
+		return GDA_TYPE_BLOB;
 	else if (!strcmp (name, "TIMESTAMP"))
-		return GDA_VALUE_TYPE_TIMESTAMP;
+		return GDA_TYPE_TIMESTAMP;
 	else if (!strcmp (name, "TIME"))
-		return GDA_VALUE_TYPE_TIME;
+		return GDA_TYPE_TIME;
 	else if (!strcmp (name, "DATE"))
-		return GDA_VALUE_TYPE_DATE;
+		return G_TYPE_DATE;
 	else if (!strcmp (name, "INT64"))
-		return GDA_VALUE_TYPE_NUMERIC;
+		return GDA_TYPE_NUMERIC;
 	else if (!strcmp (name, "FLOAT"))
-		return GDA_VALUE_TYPE_SINGLE;
+		return G_TYPE_FLOAT;
 	else if (!strcmp (name, "DOUBLE"))
-		return GDA_VALUE_TYPE_DOUBLE;
+		return G_TYPE_DOUBLE;
 
-	return GDA_VALUE_TYPE_STRING;
+	return G_TYPE_STRING;
 }
 
 
@@ -436,18 +426,16 @@ fb_set_index_field_metadata (GdaConnection *cnc,
 			     GdaDataModel *recset,
 			     const gchar *table_name)
 {
-	GdaTransaction *transaction;
 	GdaCommand *command;
 	gchar *sql, *field_name, *recset_field_name;
 	gint i,j;
-	GList *reclist;
-	GdaValue *value, *recset_value;
+	GValue *value, *recset_value;
 	
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 	g_return_if_fail (GDA_IS_DATA_MODEL (recset));
 
-	transaction = gda_transaction_new ("tmp_transaction_mdata");
-	if (gda_connection_begin_transaction (cnc, transaction)) {
+	if (gda_connection_begin_transaction (cnc, "tmp_transaction_mdata", GDA_TRANSACTION_ISOLATION_UNKNOWN, NULL)) {
+		GdaDataModel *recmodel;
 		sql = g_strdup_printf (
 				"SELECT A.RDB$FIELD_NAME, I.RDB$UNIQUE_FLAG, B.RDB$CONSTRAINT_TYPE "
 				"FROM RDB$INDEX_SEGMENTS A, RDB$RELATION_CONSTRAINTS B, RDB$INDICES I "
@@ -457,63 +445,61 @@ fb_set_index_field_metadata (GdaConnection *cnc,
 				"ORDER BY A.RDB$FIELD_POSITION",
 				table_name);
 		command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
-		gda_command_set_transaction (command, transaction);
-		reclist = gda_connection_execute_command (cnc, command, NULL, NULL);
-		if (reclist) {
+		recmodel = gda_connection_execute_select_command (cnc, command, NULL, NULL);
+		if (recmodel) {
 			gda_data_model_freeze (recset);
 			
 			/* For each index or unique field in statement */
-			for (i = 0; i < gda_data_model_get_n_rows (GDA_DATA_MODEL (reclist->data)); i++) {
-				value = (GdaValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data), 0, i);
-				field_name = (gchar *) gda_value_get_string (value);
+			for (i = 0; i < gda_data_model_get_n_rows (recmodel); i++) {
+				value = (GValue *) gda_data_model_get_value_at (recmodel, 0, i);
+				field_name = (gchar *) g_value_get_string (value);
 				j = -1;
 
 				/* Find column in @recset by name */
 				do {
 					j++;
-					recset_value = (GdaValue *) gda_data_model_get_value_at (recset, 0, j);
-					recset_field_name = (gchar *) gda_value_get_string (recset_value);
-				} while (strcmp (recset_field_name, field_name) && (j < gda_data_model_get_n_rows (recset)));
+					recset_value = (GValue *) gda_data_model_get_value_at (recset, 0, j);
+					recset_field_name = (gchar *) g_value_get_string (recset_value);
+				} while (strcmp (recset_field_name, field_name) && 
+					 (j < gda_data_model_get_n_rows (recset)));
 				
 				/* Set recset values */
 				if (!strcmp (recset_field_name, field_name)) {
 					
 					/* Get primary key value from statement */
-					value = (GdaValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data),
+					value = (GValue *) gda_data_model_get_value_at (recmodel,
 											  2,
 											  i);
 					
-					/* Get recorset GdaValue for primary key (position 5) */
-					recset_value = (GdaValue *) gda_data_model_get_value_at (recset, 5, j);
+					/* Get recorset GValue for primary key (position 5) */
+					recset_value = (GValue *) gda_data_model_get_value_at (recset, 5, j);
 
 					/* Set Primary Key mark */
-					gda_value_set_boolean (recset_value, (!strcmp (
-										gda_value_get_string (value),
+					g_value_set_boolean (recset_value, (!strcmp (
+										g_value_get_string (value),
 										"PRIMARY KEY")));
 					
 					/* Get unique index value from statement */
-					value = (GdaValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data),
+					value = (GValue *) gda_data_model_get_value_at (recmodel,
 											  1,
 											  i);
 					
-					/* Get recorset GdaValue for unique index (position 6) */
-					recset_value = (GdaValue *) gda_data_model_get_value_at (recset, 6, j);
+					/* Get recorset GValue for unique index (position 6) */
+					recset_value = (GValue *) gda_data_model_get_value_at (recset, 6, j);
 
 					/* Set Unique Index mark */
-					gda_value_set_boolean (recset_value, (gda_value_get_smallint (value) == 1));
+					g_value_set_boolean (recset_value, (gda_value_get_short (value) == 1));
 				}
 			}
 			
 			gda_data_model_thaw (recset);
-			g_list_free (reclist);
+			g_object_unref (recmodel);
 		}
 		
 		g_free (sql);
-		gda_connection_rollback_transaction (cnc, transaction);
-		gda_command_free (command);						
+		gda_connection_rollback_transaction (cnc, "tmp_transaction_mdata", NULL);
+		gda_command_free (command);
 	}
-
-	g_object_unref (transaction);
 }
 
 
@@ -523,90 +509,78 @@ fb_set_index_field_metadata (GdaConnection *cnc,
  *  Fill a list with metadata values from @row.
  *  This function is called from #fb_get_fields_metadata.
  *
- *  Returns: A GList of GdaValues
+ *  Returns: A GList of GValues
  */
 static GList *
 fb_set_field_metadata (GdaRow *row)
 {
 	GList *value_list = NULL;
-	GdaValue *value;
+	GValue *value;
 	gchar *the_value, *tmp_value;
 	gshort scale, short_value;
+	GValue *tmpval;
+	GType type;
 
 	g_return_val_if_fail (row != NULL, NULL);
 
 	/* Set field name  */
 	value = gda_row_get_value (row, 0);
-	the_value = (gchar *) gda_value_get_string (value);
-	value_list = g_list_append (value_list,	gda_value_new_string (the_value));
-	
-	/* Get Scale (number of decimals) */
-	value = gda_row_get_value (row, 4);
-	if (!gda_value_is_null (value))
-		scale = gda_value_get_smallint (value);
-	else
-		scale = 0;
-	
+	value_list = g_list_append (value_list,	gda_value_copy (value));
+		
 	/* Set data type */
 	value = gda_row_get_value (row, 1);
-	tmp_value = (gchar *) gda_value_get_string (value);
+	tmp_value = (gchar *) g_value_get_string (value);
 	tmp_value = g_strchomp (tmp_value);
 	the_value = g_ascii_strdown (tmp_value, -1);
 	g_free (tmp_value);
-	
-	/* Set correct value name */
-	if (!strcmp (the_value, "long") && (scale < 0))
-		value_list = g_list_append (value_list, gda_value_new_string ("decimal"));
-	else if (!strcmp (the_value, "long"))
-		value_list = g_list_append (value_list, gda_value_new_string ("integer"));
-	else if (!strcmp (the_value, "int64"))
-		value_list = g_list_append (value_list, gda_value_new_string ("numeric"));
-	else if (!strcmp (the_value, "varying"))
-		value_list = g_list_append (value_list, gda_value_new_string ("varchar"));
-	else if (!strcmp (the_value, "text"))
-		value_list = g_list_append (value_list, gda_value_new_string ("char"));
-	else if (!strcmp (the_value, "short"))
-		value_list = g_list_append (value_list, gda_value_new_string ("smallint"));
-	else if (!strcmp (the_value, "double"))
-		value_list = g_list_append (value_list, gda_value_new_string ("double precision"));
-	else
-		value_list = g_list_append (value_list, gda_value_new_string (the_value));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), the_value);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* Set size */
 	short_value = 0;
-	if (fb_type_name_to_gda_type (the_value) == GDA_VALUE_TYPE_STRING) {
+	if (fb_type_name_to_g_type (the_value) == G_TYPE_STRING) {
 		value = gda_row_get_value (row, 2);
-		short_value = gda_value_get_smallint (value);
+		short_value = gda_value_get_short (value);
 	}
-	else if (fb_type_name_to_gda_type (the_value) == GDA_VALUE_TYPE_NUMERIC) {
+	else if (fb_type_name_to_g_type (the_value) == GDA_TYPE_NUMERIC) {
 		value = gda_row_get_value (row, 3);
-		short_value = gda_value_get_smallint (value);
+		short_value = gda_value_get_short (value);
 	}
-	value_list = g_list_append (value_list, gda_value_new_integer (short_value));
+	g_value_set_int (tmpval = gda_value_new (G_TYPE_INT), short_value);
+	value_list = g_list_append (value_list, tmpval);
 
-	/* Set scale */	
-	value_list = g_list_append (value_list, gda_value_new_integer ((scale * -1)));
+	/* Set scale (number of decimals) */
+	value = gda_row_get_value (row, 4);
+	if (!gda_value_is_null (value))
+		scale = gda_value_get_short (value);
+	else
+		scale = 0;
+	g_value_set_int (tmpval = gda_value_new (G_TYPE_INT), scale * -1);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* Set not null indicator */
 	value = gda_row_get_value (row, 5);
 	if (!gda_value_is_null (value))
-		value_list = g_list_append (
-				value_list,
-				gda_value_new_boolean ((gda_value_get_smallint (value) == 1)));
+		g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), (gda_value_get_short (value) == 1));
 	else
-		value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
+		g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), FALSE);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* Primary key ? (real value set in #fb_set_index_field_metadata) */
-	value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
+	g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), FALSE);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* Unique key ? (real value set in #fb_set_index_field_metadata) */
-	value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
+	g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), FALSE);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* FIXME: References */
-	value_list = g_list_append (value_list, gda_value_new_string (""));
+	tmpval = gda_value_new_null ();
+	value_list = g_list_append (value_list, tmpval);
 
 	/* FIXME: Default value */
-	value_list = g_list_append (value_list, gda_value_new_string (""));
+	tmpval = gda_value_new_null ();
+	value_list = g_list_append (value_list, tmpval);
 
 	return value_list;
 }
@@ -624,7 +598,6 @@ static GdaDataModel *
 fb_get_fields_metadata (GdaConnection *cnc,
 			GdaParameterList *params)
 {
-	GdaTransaction *transaction;
 	GList *value_list, *reclist;
 	GdaRow *row;
 	GdaDataModelArray *recset = NULL;
@@ -635,17 +608,17 @@ fb_get_fields_metadata (GdaConnection *cnc,
 	const gchar *table_name;
 	struct {
 	  	const gchar *name;
-		GdaValueType type;
+		GType type;
 	} fields_desc[] = {
-		{ N_("Field name")      , GDA_VALUE_TYPE_STRING  },
-		{ N_("Data type")       , GDA_VALUE_TYPE_STRING  },
-		{ N_("Size")            , GDA_VALUE_TYPE_INTEGER },
-		{ N_("Scale")           , GDA_VALUE_TYPE_INTEGER },
-		{ N_("Not null?")       , GDA_VALUE_TYPE_BOOLEAN },
-		{ N_("Primary key?")    , GDA_VALUE_TYPE_BOOLEAN },
-		{ N_("Unique index?")   , GDA_VALUE_TYPE_BOOLEAN },
-		{ N_("References")      , GDA_VALUE_TYPE_STRING  },
-		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  }
+		{ N_("Field name")      , G_TYPE_STRING  },
+		{ N_("Data type")       , G_TYPE_STRING  },
+		{ N_("Size")            , G_TYPE_INT },
+		{ N_("Scale")           , G_TYPE_INT },
+		{ N_("Not null?")       , G_TYPE_BOOLEAN },
+		{ N_("Primary key?")    , G_TYPE_BOOLEAN },
+		{ N_("Unique index?")   , G_TYPE_BOOLEAN },
+		{ N_("References")      , G_TYPE_STRING  },
+		{ N_("Default value")   , G_TYPE_STRING  }
 	};
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -658,15 +631,15 @@ fb_get_fields_metadata (GdaConnection *cnc,
 		return NULL;
 	}
 
-	table_name = gda_value_get_string ((GdaValue *) gda_parameter_get_value (par));
+	table_name = g_value_get_string ((GValue *) gda_parameter_get_value (par));
 	if (!table_name) {
 		gda_connection_add_event_string (cnc,
 				_("Table name is needed but none specified in parameter list"));
 		return NULL;
 	}
 	
-	transaction = gda_transaction_new ("temp_transaction");
-	if (gda_connection_begin_transaction (cnc, transaction)) {
+	if (gda_connection_begin_transaction (cnc, "temp_transaction", GDA_TRANSACTION_ISOLATION_UNKNOWN, NULL)) {
+		GdaDataModel *recmodel;
 		sql = g_strdup_printf (
 				"SELECT A.RDB$FIELD_NAME, C.RDB$TYPE_NAME, B.RDB$FIELD_LENGTH, "
 				"B.RDB$FIELD_PRECISION, B.RDB$FIELD_SCALE, A.RDB$NULL_FLAG "
@@ -676,9 +649,8 @@ fb_get_fields_metadata (GdaConnection *cnc,
 				" ORDER BY A.RDB$FIELD_POSITION",
 				table_name);
 		command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
-		gda_command_set_transaction (command, transaction);
-		reclist = gda_connection_execute_command (cnc, command, NULL, NULL);
-		if (reclist) {
+		recmodel = gda_connection_execute_select_command (cnc, command, NULL, NULL);
+		if (recmodel) {
 			/* Create and fill recordset to be returned */
 			recset = (GdaDataModelArray *) gda_data_model_array_new (9);
 			for (i = 0; i < (sizeof (fields_desc) / sizeof (fields_desc[0])); i++)
@@ -699,16 +671,14 @@ fb_get_fields_metadata (GdaConnection *cnc,
 			}
 			
 			fb_set_index_field_metadata (cnc, (GdaDataModel *) recset, table_name);
-			g_list_free (reclist);
+			g_object_unref (recmodel);
 		}
 
 		g_free (sql);
-		gda_connection_rollback_transaction (cnc, transaction);
+		gda_connection_rollback_transaction (cnc, "temp_transaction", NULL);
 		gda_command_free (command);						
 	}
 
-	g_object_unref (transaction);
-
 	return GDA_DATA_MODEL (recset);
 }
 
@@ -718,23 +688,31 @@ fb_add_aggregate_row (GdaDataModelArray *recset,
 		      const gchar *comments)
 {
 	GList *list;
+	GValue *tmpval;
 
 	g_return_if_fail (GDA_IS_DATA_MODEL_ARRAY (recset));
 
 	/* 1st the name */
-	list = g_list_append (NULL, gda_value_new_string (str));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), str);
+	list = g_list_append (NULL, tmpval);
 	/* 2nd the unique id */
-	list = g_list_append (list, gda_value_new_string (str));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), str);
+	list = g_list_append (list, tmpval);
 	/* 3rd the owner */
-	list = g_list_append (list, gda_value_new_string (NULL));
+	tmpval = gda_value_new_null ();
+	list = g_list_append (list, tmpval);
 	/* 4th the comments */
-	list = g_list_append (list, gda_value_new_string (comments));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), comments);
+	list = g_list_append (list, tmpval);
 	/* 5th the return type */
-	list = g_list_append (list, gda_value_new_string (_("UNKNOWN")));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), _("UNKNOWN"));
+	list = g_list_append (list, tmpval);
 	/* 6th the argument type */
-	list = g_list_append (list, gda_value_new_string (_("UNKNOWN")));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), _("UNKNOWN"));
+	list = g_list_append (list, tmpval);
 	/* 7th the SQL definition */
-	list = g_list_append (list, gda_value_new_string (NULL));
+	tmpval = gda_value_new_null ();
+	list = g_list_append (list, tmpval);
 
 	gda_data_model_append_values (GDA_DATA_MODEL (recset), list, NULL);
 
@@ -1100,7 +1078,7 @@ gda_firebird_provider_run_sql (GList *reclist,
 	recset = NULL;
 
 	/* parse SQL string, which can contain several commands, separated by ';' */
-	arr = g_strsplit (sql, ";", 0);
+	arr = gda_delimiter_split_sql (sql);
 	if (arr) {
 		while (arr[n]) {
 			recset = gda_firebird_recordset_new (cnc, ftr, sql);
@@ -1164,30 +1142,23 @@ gda_firebird_provider_execute_command (GdaServerProvider *provider,
 	isc_tr_handle *ftr;
 	GList *reclist = NULL;
 	gboolean commit_tr;
-	GdaTransaction *xaction;
 
 	g_return_val_if_fail (GDA_IS_FIREBIRD_PROVIDER (provider), NULL);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (cmd != NULL, NULL);
 	
 	commit_tr = FALSE;
-
-	/* Get transaction */
-	xaction = gda_command_get_transaction (cmd);
-	if (!GDA_IS_TRANSACTION (xaction))
-		xaction = gda_transaction_new ("local_tr");
 	
 	/* Get transaction handle */
-	ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
+	ftr = g_object_get_data (G_OBJECT (cnc), TRANSACTION_DATA);
 	
 	/* Begin transaction if it was not already started  */
 	if (!ftr) {
-
 		/* Start transaction */
-		commit_tr = gda_firebird_provider_begin_transaction (provider, cnc, xaction);
+		commit_tr = gda_firebird_provider_begin_transaction (provider, cnc, "local_tr", GDA_TRANSACTION_ISOLATION_UNKNOWN, NULL);
 	
 		/* Get transaction handle */
-		ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
+		ftr = g_object_get_data (G_OBJECT (cnc), TRANSACTION_DATA);
 	}
 	
 	/* Parse command */
@@ -1200,10 +1171,8 @@ gda_firebird_provider_execute_command (GdaServerProvider *provider,
 	}
 
 	/* Commit transaction if it was not started directly by programmer */
-	if (commit_tr) {
-		gda_firebird_provider_commit_transaction (provider, cnc, xaction);
-		g_object_unref (xaction);
-	}
+	if (commit_tr) 
+		gda_firebird_provider_commit_transaction (provider, cnc, "local_tr", NULL);
 	
 	return reclist;
 }
@@ -1212,7 +1181,8 @@ gda_firebird_provider_execute_command (GdaServerProvider *provider,
 static gboolean
 gda_firebird_provider_begin_transaction (GdaServerProvider *provider,
 					 GdaConnection *cnc,
-					 GdaTransaction *xaction)
+					 const gchar *name, GdaTransactionIsolation level,
+					 GError **error)
 {
 	GdaFirebirdConnection *fcnc;
 	isc_tr_handle *ftr;
@@ -1243,7 +1213,7 @@ gda_firebird_provider_begin_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 	
-	g_object_set_data (G_OBJECT (xaction), TRANSACTION_DATA, ftr);
+	g_object_set_data (G_OBJECT (cnc), TRANSACTION_DATA, ftr);
 
 	return TRUE;
 }
@@ -1252,7 +1222,7 @@ gda_firebird_provider_begin_transaction (GdaServerProvider *provider,
 static gboolean
 gda_firebird_provider_commit_transaction (GdaServerProvider *provider,
 					  GdaConnection *cnc,
-					  GdaTransaction *xaction)
+					  const gchar *name, GError **error)
 {
 	GdaFirebirdConnection *fcnc;
 	isc_tr_handle *ftr;
@@ -1267,7 +1237,7 @@ gda_firebird_provider_commit_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 
-	ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
+	ftr = g_object_get_data (G_OBJECT (cnc), TRANSACTION_DATA);
 	if (!ftr) {
 		gda_connection_add_event_string (cnc, _("Invalid transaction handle"));
 		return FALSE;
@@ -1281,7 +1251,7 @@ gda_firebird_provider_commit_transaction (GdaServerProvider *provider,
 		result = TRUE;
 
 	g_free (ftr);
-	g_object_set_data (G_OBJECT (xaction), TRANSACTION_DATA, NULL);
+	g_object_set_data (G_OBJECT (cnc), TRANSACTION_DATA, NULL);
 
 	return result;
 }
@@ -1290,7 +1260,7 @@ gda_firebird_provider_commit_transaction (GdaServerProvider *provider,
 static gboolean
 gda_firebird_provider_rollback_transaction (GdaServerProvider *provider,
 					    GdaConnection *cnc,
-					    GdaTransaction *xaction)
+					    const gchar *name, GError **error)
 {
 	GdaFirebirdConnection *fcnc;
 	isc_tr_handle *ftr;
@@ -1305,7 +1275,7 @@ gda_firebird_provider_rollback_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 
-	ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
+	ftr = g_object_get_data (G_OBJECT (cnc), TRANSACTION_DATA);
 	if (!ftr) {
 		gda_connection_add_event_string (cnc, _("Invalid transaction handle"));
 		return FALSE;
@@ -1319,7 +1289,7 @@ gda_firebird_provider_rollback_transaction (GdaServerProvider *provider,
 		result = TRUE;
 
 	g_free (ftr);
-	g_object_set_data (G_OBJECT (xaction), TRANSACTION_DATA, NULL);
+	g_object_set_data (G_OBJECT (cnc), TRANSACTION_DATA, NULL);
 
 	return result;
 }
@@ -1385,13 +1355,3 @@ gda_firebird_provider_get_schema (GdaServerProvider *provider,
 
 	return NULL;
 }
-
-
-static GdaBlob *
-gda_firebird_provider_create_blob (GdaServerProvider *provider, GdaConnection *cnc)
-{
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-
-	return gda_firebird_blob_new (cnc);
-}
