@@ -80,7 +80,7 @@ struct _GdaIBMDB2Field {
 };
 																		
 
-GType         gda_ibmdb2_recordset_get_type (void) G_GNUC_CONST;
+GType         gda_ibmdb2_recordset_get_type (void);
 GdaDataModel *gda_ibmdb2_recordset_new (GdaConnection *cnc, SQLHANDLE hstmt);
 
 G_END_DECLS
