@@ -1,5 +1,5 @@
 /* GDA Oracle provider
- * Copyright (C) 2002 The GNOME Foundation.
+ * Copyright (C) 2002 - 2004 The GNOME Foundation.
  *
  * AUTHORS:
  * 	Tim Coleman <tim@timcoleman.com>
@@ -25,8 +25,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if !defined(__gda_oracle_recordset_h__)
-#  define __gda_oracle_recordset_h__
+#ifndef __GDA_ORACLE_RECORDSET_H__
+#define __GDA_ORACLE_RECORDSET_H__
 
 #include <libgda/gda-data-model-hash.h>
 #include <libgda/gda-value.h>
@@ -48,12 +48,12 @@ typedef struct _GdaOracleRecordsetPrivate GdaOracleRecordsetPrivate;
 typedef struct _GdaOracleValue            GdaOracleValue;
 
 struct _GdaOracleRecordset {
-	GdaDataModel model;
+	GdaDataModelRow model;
 	GdaOracleRecordsetPrivate *priv;
 };
 
 struct _GdaOracleRecordsetClass {
-	GdaDataModelClass parent_class;
+	GdaDataModelRowClass parent_class;
 };
 
 struct _GdaOracleValue {
@@ -63,12 +63,13 @@ struct _GdaOracleValue {
 	ub2 sql_type;
 	ub2 defined_size;
 	gpointer value;
-	GdaValueType gda_type;
+	GType g_type;
 };
 
 
 GType               gda_oracle_recordset_get_type (void);
-GdaOracleRecordset *gda_oracle_recordset_new (GdaConnection *cnc, GdaOracleConnectionData *cdata, OCIStmt *stmthp);
+GdaOracleRecordset *gda_oracle_recordset_new (GdaConnection *cnc, GdaOracleConnectionData *cdata, 
+					      OCIStmt *stmthp, GArray *col_size_array);
 
 G_END_DECLS
 
