@@ -1,8 +1,9 @@
 /* GDA - Command line configuration utility
- * Copyright (C) 2002 The GNOME Foundation.
+ * Copyright (C) 2002 - 2005 The GNOME Foundation.
  *
  * AUTHORS:
  * 	Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,14 +24,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
-#include <popt.h>
 #include <libgda/libgda.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <readline/readline.h>
 #include <readline/history.h>
 
-
 #define PRINT_ALL	0
 #define PRINT_SECTIONS	1
 #define PRINT_ENTRIES	2
@@ -54,6 +53,9 @@ typedef struct {
 	mode_function function;
 } cmd;
 
+/* GOption callback prototypes */
+static gboolean list_providers (const gchar *option_name, const gchar *value, gpointer data, GError **error);
+static gboolean list_datasources (const gchar *option_name, const gchar *value, gpointer data, GError **error);
 /*
  * Static variables
  */
@@ -65,8 +67,6 @@ static const gchar *prompt_config = "config> ";
 enum {
 	ACTION_NONE = 0,
 	ACTION_FILE = 1,
-	ACTION_LIST_PROVIDERS = 1 << 1,
-	ACTION_LIST_DATASOURCES = 1 << 2
 };
 
 typedef struct {
@@ -340,7 +340,7 @@ parse_config_file (const gchar *buffer, int len)
 		return NULL;
 	}
 
-	if (strcmp (cur->name, "libgda-config") != 0){
+	if (strcmp ((gchar*)cur->name, "libgda-config") != 0){
 		g_error ("root node != 'libgda-config' -> '%s'", cur->name);
 		xmlFreeDoc (doc);
 		return NULL;
@@ -350,7 +350,7 @@ parse_config_file (const gchar *buffer, int len)
 	while (cur != NULL) {
 		if (!strcmp(cur->name, "section")){
 			section = g_new0 (gda_config_section, 1);
-			section->path = xmlGetProp (cur, "path");
+			section->path = xmlGetProp(cur, (xmlChar*)"path");
 			if (section->path != NULL){
 				section->entries = config_read_entries (cur);
 				list = g_list_append (list, section);
@@ -404,7 +404,7 @@ static gboolean
 config_load_file (const gchar *file)
 {
 	gchar *file_contents;
-	gint len;
+	gsize len;
 
 	if (g_file_get_contents (file, &file_contents, &len, NULL)){
 		config_data = parse_config_file (file_contents, len);
@@ -957,9 +957,9 @@ add_xml_entry (xmlNodePtr parent, gda_config_entry *entry)
 	xmlNodePtr new_node;
 
 	new_node = xmlNewTextChild (parent, NULL, "entry", NULL);
-	xmlSetProp (new_node, "name", entry->name);
-	xmlSetProp (new_node, "type", entry->type);
-	xmlSetProp (new_node, "value", entry->value);
+	xmlSetProp(new_node, (xmlChar*)"name", entry->name);
+	xmlSetProp(new_node, (xmlChar*)"type", entry->type);
+	xmlSetProp(new_node, (xmlChar*)"value", entry->value);
 }
 
 static xmlNodePtr
@@ -968,7 +968,7 @@ add_xml_section (xmlNodePtr parent, gda_config_section *section)
 	xmlNodePtr new_node;
 
 	new_node = xmlNewTextChild (parent, NULL, "section", NULL);
-	xmlSetProp (new_node, "path", section->path);
+	xmlSetProp(new_node, (xmlChar*)"path", section->path);
 	return new_node;
 }
 
@@ -1189,94 +1189,99 @@ initialize_readline ()
  */
 
 static void 
-options (int argc, const char **argv)
+options (gint argc, const gchar **argv)
 {
-	poptContext context;
+
+	GOptionContext *context;
+	GError *error = NULL;
 	
-	static const struct poptOption options[] = {
+	static const GOptionEntry options[] = {
 		{
 			"config-file", 
 			'c', 
-			POPT_ARG_STRING, 
+			0,
+			G_OPTION_ARG_FILENAME, 
 			&cmdArgs.config_file, 
-			0, 
 			"File to load the configuration from.",
 			"filename"
 		},
 		{
 			"list-providers",
 			'l',
-			POPT_ARG_VAL | POPT_ARGFLAG_OR,
-			&cmdArgs.actions,
-			ACTION_LIST_PROVIDERS,
-			"Lists installed providers"
+			G_OPTION_FLAG_NO_ARG,
+			G_OPTION_ARG_CALLBACK,
+			list_providers,
+			"Lists installed providers",
+			NULL
 		},
 		{
 			"list-datasources",
 			'L',
-			POPT_ARG_VAL | POPT_ARGFLAG_OR,
-			&cmdArgs.actions,
-			ACTION_LIST_DATASOURCES,
-			"Lists configured datasources"
+			G_OPTION_FLAG_NO_ARG,
+			G_OPTION_ARG_CALLBACK,
+			list_datasources,
+			"Lists configured datasources",
+			NULL
 		},
 		{
 			"name",
 			'n',
-			POPT_ARG_STRING,
-			&cmdArgs.name,
 			0,
-			"User-assigned name for this connection."
+			G_OPTION_ARG_STRING,
+			&cmdArgs.name,
+			"User-assigned name for this connection.",
+			"STRING"
 		},
 		{
 			"user",
 			'u',
-			POPT_ARG_STRING,
-			&cmdArgs.user,
 			0,
-			"User name to pass to the provider."
+			G_OPTION_ARG_STRING,
+			&cmdArgs.user,
+			"User name to pass to the provider.",
+			"STRING"
 		},
 		{
 			"password",
 			'p',
-			POPT_ARG_STRING,
-			&cmdArgs.password,
 			0,
+			G_OPTION_ARG_STRING,
+			&cmdArgs.password,
 			"Password for the given user to connect to the DB "
-			"backend."
+			"backend.",
+			"STRING"
 		},
 		{
 			"provider",
 			'P',
-			POPT_ARG_STRING,
-			&cmdArgs.provider,
 			0,
-			"Provider name."
+			G_OPTION_ARG_STRING,
+			&cmdArgs.provider,
+			"Provider name.",
+			"STRING"
 		},
 		{
 			"DSN",
 			'd',
-			POPT_ARG_STRING,
-			&cmdArgs.dsn,
 			0,
+			G_OPTION_ARG_STRING,
+			&cmdArgs.dsn,
 			"Semi-colon separated string with name=value options "
-			"to pass to the GDA provider."
+			"to pass to the GDA provider.",
+			"STRING"
 		},
-		POPT_AUTOHELP
-		{NULL, '\0', 0, NULL, 0}
+		{ NULL }
 	};
 
-	context = poptGetContext (NULL, argc, argv, options, 0);
-	if (!context)
-		return;
-
-	if (poptGetNextOpt (context) < -1 || poptGetArg (context) != NULL){
-		g_print ("-Error: Incorrect argument in command line.\n");
-		poptPrintHelp (context, stderr, 0);
-		poptFreeContext (context);
+	context = g_option_context_new ("");
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_parse (context, &argc, (gchar ***) &argv, &error);
+	if (error != NULL) {
+		g_print ("-Error: %s\n", error->message);
+		g_error_free (error);
 		exit (1);
 	}
-
-	poptFreeContext (context);
+	g_option_context_free (context);
 
 	if (!cmdArgs.name && (cmdArgs.user || 
 			      cmdArgs.dsn  || 
@@ -1288,7 +1293,7 @@ options (int argc, const char **argv)
 
 	if (cmdArgs.name != NULL) {
 		if (cmdArgs.actions != ACTION_NONE) {
-			g_print ("-Error: cannot use -n in this context.\n");
+			g_print ("-Error: Cannot use -n in this context.\n");
 			exit (1);
 		}
 		cmdArgs.actions |= ACTION_FILE;
@@ -1378,16 +1383,17 @@ batch_options ()
 static void
 add_param_name_to_string (gpointer data, gpointer user_data)
 {
-	GdaProviderParameterInfo *param_info = data;
+	GdaParameter *param = GDA_PARAMETER (data);
 	GString *str = user_data;
+	gchar *tmp;
 
-	if (param_info != NULL) {
-		g_string_append_c (str, ' ');
-		g_string_append (str, param_info->name);
-	}
+	g_object_get (G_OBJECT (param), "string_id", &tmp, NULL);
+	g_string_append_c (str, ' ');
+	g_string_append (str, tmp);
+	g_free (tmp);
 }
 
-static void
+static gboolean
 display_provider (gpointer data, gpointer user_data)
 {
 	GdaProviderInfo *info = data;
@@ -1400,15 +1406,21 @@ display_provider (gpointer data, gpointer user_data)
 	pretty_print (info->description, strlen (desc));
 	g_print (paramInDsn);
 	str = g_string_new (NULL);
-	g_list_foreach (info->gda_params, add_param_name_to_string, str);
+	if (info->gda_params) 
+		g_slist_foreach (info->gda_params->parameters, add_param_name_to_string, str);
+	else
+		g_string_append (str, "No DSN (or wrong) DSN spec");
 	pretty_print (str->str, strlen (paramInDsn) + 1);
 	g_string_free (str, TRUE);
 	g_print ("Location: %s\n", info->location);
 	g_print ("---\n");
 }
 
-static void
-list_providers ()
+static gboolean
+list_providers (const gchar *option_name,
+                const gchar *value,
+                gpointer data,
+                GError **error)
 {
 	GList *providers;
 	
@@ -1423,7 +1435,8 @@ list_providers ()
 	}
 	
 	g_print ("---Providers list---\n");
-	g_list_foreach (providers, display_provider, NULL);
+	g_list_foreach (providers, (GFunc) display_provider, NULL);
+	exit (0);
 }
 
 static void
@@ -1445,8 +1458,11 @@ display_datasource (gpointer data, gpointer user_data)
 	g_print ("---\n");
 }
 
-static void
-list_datasources ()
+static gboolean
+list_datasources (const gchar *option_name,
+                  const gchar *value,
+                  gpointer data,
+                  GError **error)
 {
 	GList *datasources;
 
@@ -1463,26 +1479,23 @@ list_datasources ()
 	g_print("---Datasources list---\n");
 	g_list_foreach (datasources, display_datasource, NULL);
 	gda_config_free_data_source_list (datasources);
+	exit (0);
 }
 
 /*
  * main
  */
 int
-main (int argc, char *argv [])
+main (gint argc, gchar **argv)
 {
-	options (argc, (const char **) argv);
+	options (argc, (const gchar **) argv);
+	gda_init ("Gda configuration manager", PACKAGE_VERSION, argc, argv);
+
 	if (cmdArgs.actions == ACTION_NONE) {
 		g_print ("Using configuration file from %s\n", cmdArgs.config_file);
 		initialize_readline ();
 		mode_gda ();
 	} else {
-		if ((cmdArgs.actions & ACTION_LIST_PROVIDERS) != 0)
-			list_providers ();
-
-		if ((cmdArgs.actions & ACTION_LIST_DATASOURCES) != 0)
-			list_datasources ();
-
 		if ((cmdArgs.actions & ACTION_FILE) != 0)
 			batch_options ();
 
